/***************************************************************************
 *   Copyright (C) 2007-2008 by Peter Semiletov                            *
 *   peter.semiletov@gmail.com                                             *

08 November 2007
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QtGui>
#include <QtScript>
#include <QHttp>

#ifdef Q_WS_WIN
#include <windows.h>
#endif


#include "rvln.h"
#include "utils.h"

#ifdef ASPELL_ENABLE

#include "spellchecker.h"

#endif

#include "norwegianwoodstyle.h"


void rvln::create_paths ()
{

  QDir dr;
  dir_config = dr.homePath ();

#ifdef Q_WS_WIN

  dir_config.append ("/tea");

#else

  dir_config.append ("/.config/tea");
  qDebug () << dir_config;

#endif

  dr.setPath (dir_config);
  if (! dr.exists ())
     dr.mkpath (dir_config);

  fname_crapbook = dir_config + "/crapbook.txt";
  fname_compdict = dir_config + "/compdict.txt";
  fname_bookmarks = dir_config + "/tea_bmx";
  fname_programs = dir_config + "/programs";
  fname_places_bookmarks = dir_config + "/places_bookmarks";

  fname_tempfile = QDir::tempPath() + "/tea.tmp";
  fname_tempparamfile = QDir::tempPath() + "/teaparam.tmp";

  dir_templates = dir_config + "/templates";

  dr.setPath (dir_templates);
  if (! dr.exists ())
     dr.mkpath (dir_templates);

  dir_snippets = dir_config + "/snippets";

  dr.setPath (dir_snippets);
  if (! dr.exists ())
     dr.mkpath (dir_snippets);

  dir_scripts = dir_config + "/scripts";

  dr.setPath (dir_scripts);
  if (! dr.exists ())
     dr.mkpath (dir_scripts);

  dir_sessions = dir_config + "/sessions";

  dr.setPath (dir_sessions);
  if (! dr.exists ())
     dr.mkpath (dir_sessions);
}


void rvln::update_bookmarks()
{
  if (! file_exists (fname_bookmarks))
     return;

  bookmarks = qstring_load (fname_bookmarks);
  if (bookmarks.isEmpty())
     return;

  menu_file_bookmarks->clear();
  create_menu_from_list (this, menu_file_bookmarks,
                         bookmarks.split ("\n"),
                         SLOT (file_open_bookmark()));
}


void rvln::readSettings()
{
  markup_mode = settings->value ("markup_mode", "HTML").toString();
  charset = settings->value ("charset", "UTF-8").toString();
  QPoint pos = settings->value ("pos", QPoint(1, 200)).toPoint();
  QSize size = settings->value ("size", QSize(1, 400)).toSize();
  mainSplitter->restoreState (settings->value("splitterSizes").toByteArray());
  resize (size);
  move (pos);
}


void rvln::writeSettings()
{
  settings->setValue ("pos", pos());
  settings->setValue ("size", size());
  settings->setValue ("charset", charset);
  settings->setValue ("splitterSizes", mainSplitter->saveState());

#ifndef Q_WS_WIN


  settings->setValue ("spl_fman", spl_fman->saveState());

#endif

  settings->setValue ("icons_size", iconSize().width());
  settings->setValue ("dir_last", dir_last);
  settings->setValue ("markup_mode", markup_mode);
  settings->setValue ("VERSION_NUMBER", QString (VERSION_NUMBER));

//  settings->setValue ("infpos", inf->pos());

  settings->setValue ("state", saveState());

  delete settings;
}


void rvln::create_main_widget()
{
  main_tab_widget = new QTabWidget;
  main_tab_widget->setTabPosition (QTabWidget::East);
  main_tab_widget->setTabShape (QTabWidget::Triangular);

  tab_widget = new QTabWidget;

  QPushButton *bt_close = new QPushButton ("X", this);
  connect(bt_close, SIGNAL(clicked()), this, SLOT(close_current()));
  tab_widget->setCornerWidget (bt_close);

  QWidget *hld = new QWidget (this);

  QVBoxLayout *lt_triplex = new QVBoxLayout;

  log_memo = new QTextEdit;
  log = new CLogMemo (log_memo);

  mainSplitter = new QSplitter (Qt::Vertical);

  fte = new QLineEdit;
  connect(fte, SIGNAL(returnPressed()), this, SLOT(search_find()));


  mainSplitter->addWidget(tab_widget);
  mainSplitter->addWidget(log_memo);

  QHBoxLayout *lt_fte = new QHBoxLayout;
  QToolButton *bt_find = new QToolButton (this);

  QToolButton *bt_prev = new QToolButton (this);
  bt_prev->setArrowType (Qt::LeftArrow);
  QToolButton *bt_next = new QToolButton (this);
  bt_next->setArrowType (Qt::RightArrow);

  connect(bt_find, SIGNAL(clicked()), this, SLOT(search_find()));

  bt_find->setIcon (QIcon (":/icons/search_find.svg"));

  lt_fte->addWidget (fte);
  lt_fte->addWidget (bt_find);
  lt_fte->addWidget (bt_prev);
  lt_fte->addWidget (bt_next);

  lt_triplex->addWidget (mainSplitter);
  lt_triplex->addLayout (lt_fte);

  hld->setLayout (lt_triplex);

  mainSplitter->setStretchFactor(1, 1);

  hld->show();

  main_tab_widget->addTab (hld, tr("edit"));

  setCentralWidget (main_tab_widget);
  tab_widget->resize (tab_widget->width(), width() - 100);
  connect(tab_widget, SIGNAL(currentChanged(int )), this, SLOT(pageChanged(int )));
}


void rvln::tray_activated ( QSystemTrayIcon::ActivationReason reason )
{
  //qDebug () << "tray act";
  if (reason == QSystemTrayIcon::Trigger)
     {
      if (isVisible())
         hide();
      else
          show();
     }
}


rvln::rvln()
{
  settings = new QSettings ("tea", "tea");

  if (settings->value ("override_locale", 0).toBool())
     {
      QString ts = settings->value ("override_locale_val", "en").toString();
      if (ts.length() != 2)
         ts = "en";
      QTranslator *qtTranslator  = new QTranslator;
      qtTranslator->load(QString("qt_%1").arg(ts),
                         QLibraryInfo::location(QLibraryInfo::TranslationsPath));
      qApp->installTranslator(qtTranslator);

      myappTranslator.load(":/translations/tea_" + ts);
      qApp->installTranslator(&myappTranslator);
     }
  else
  //if (! qApp->arguments().contains("--notrans"))
     {
      QTranslator *qtTranslator  = new QTranslator;
      qtTranslator->load(QString("qt_%1").arg(QLocale::system().name()),
                         QLibraryInfo::location(QLibraryInfo::TranslationsPath));
      qApp->installTranslator(qtTranslator);

      myappTranslator.load(":/translations/tea_" + QLocale::system().name());
      qApp->installTranslator(&myappTranslator);
     }

  qApp->setWindowIcon (QIcon(":/icons/tea_icon_v2.png"));

  create_paths();

  setAcceptDrops(true);

  createActions();
  createMenus();
  createToolBars();
  createStatusBar();
  update_bookmarks();
  update_templates();
  update_snippets();
  update_sessions();
  update_scripts();
  update_programs();

  update_charsets();


//  inf = new CInformer();

//  bool tvshow = settings->value ("showroxvisor", true).toBool();
  //if (tvshow)
    // inf->show();

  //inf->setVisible (tvshow);
  //inf->move (settings->value ("infpos", QPoint(x() + width() + 10, y())).toPoint());

  create_main_widget();

  readSettings();

  documents = new document_holder();
  documents->parent_wnd = this;
  documents->tab_widget = tab_widget;
  documents->recent_menu = menu_file_recent;
  documents->recent_list_fname = dir_config + "/tea_recent";
  documents->reload_recent_list();
  documents->update_recent_menu();
  documents->log = log;
  documents->status_bar = statusBar();
  documents->markup_mode = markup_mode;
  documents->fname_compdict = fname_compdict;
  documents->dir_config = dir_config;
  documents->fname_crapbook = fname_crapbook;

  shortcuts = new CShortcuts (this);
  shortcuts->fname = dir_config + "/keys";

//  KeybSound = new CKeybSound;

#ifdef ASPELL_ENABLE

  spellchecker = new CSpellchecker (settings->value("spell_lang", QLocale::system().name().left(2)).toString());
  create_spellcheck_menu();

#endif

#ifndef Q_WS_WIN

  createFman();

#endif

  createOptions();
  restoreState (settings->value("state", QByteArray()).toByteArray());
  spl_fman->restoreState (settings->value("spl_fman").toByteArray());
  createManual();

  shortcuts->load_from_file (shortcuts->fname);

  documents->settings = settings;

  QApplication::setStyle (settings->value("ui_style", "Plastique").toString());


  log->log (tr ("<b>TEA %1 @ http://tea-editor.sourceforge.net</b><br>by Peter Semiletov (peter.semiletov@gmail.com)<br>read the Manual under the <i>browse</i> tab!").arg (QString (VERSION_NUMBER)));

  updateFonts ();

  dir_last = settings->value("dir_last", QDir::homePath()).toString();

  l_status = new QLabel ();
  pb_status = new QProgressBar ();
  pb_status->setRange (0, 0);

  statusBar()->insertPermanentWidget(0, pb_status);
  statusBar()->insertPermanentWidget(1, l_status);

  pb_status->hide();
  documents->l_status_bar = l_status;

  //connect(&http, SIGNAL(responseHeaderReceived (const QHttpResponseHeader &)),
    //     this, SLOT(slot_responseHeaderReceived ( const QHttpResponseHeader &)));

  pbar_step = 0;

  connect (&timer, SIGNAL(timeout()), this, SLOT(iterate_pbar()));

/*
  tray = new QSystemTrayIcon (this);
  connect(tray, SIGNAL(activated ( QSystemTrayIcon::ActivationReason )), this, SLOT(tray_activated ( QSystemTrayIcon::ActivationReason )));

  tray->setIcon(QIcon(":/icons/tea_icon_v2.png"));
  tray->show();
*/
  //handle_commandline();

  QString vn = settings->value("VERSION_NUMBER", "").toString();
  if (vn.isEmpty() || vn != QString (VERSION_NUMBER))
    help_show_news();


  if (settings->value("session_restore", false).toBool())
     {
      QString fname_session = dir_sessions + "/" + "def-session-777";
      documents->load_from_session (fname_session);
     }
}


void rvln::iterate_pbar()
{
  pb_status->setValue (pbar_step++);
}


void rvln::closeEvent(QCloseEvent *event)
{
  QString fname = dir_config + "/last_used_charsets";
  qstring_save (fname, sl_last_used_charsets.join ("\n").trimmed());

  if (settings->value("session_restore", false).toBool())
     {
      QString fname_session = dir_sessions + "/" + "def-session-777";
      documents->save_to_session (fname_session);
     }

  writeSettings();

  delete documents;
//  delete KeybSound;
//  delete inf;
  delete log;

#ifdef ASPELL_ENABLE

  delete spellchecker;

#endif

  event->accept();
}


void rvln::newFile()
{
  CDocument *d = documents->create_new();
//  d->textEdit->installEventFilter(KeybSound);

//  inf->change_movie ("kidding02.mng");
}


#ifndef Q_WS_WIN

void rvln::open()
{
  if (! settings->value("use_trad_dialogs", "0").toBool())
    {
     main_tab_widget->setCurrentIndex (1);
     CDocument *d = documents->get_current();
     if (d)
        {
         if (file_exists (d->file_name))
             fman_nav (get_file_path (d->file_name));
         else
             fman_nav (dir_last);
         }
     else
         fman_nav (dir_last);

     return;
    }

  QFileDialog dialog (this);
  QSize size = settings->value ("dialog_size", QSize(width(), height())).toSize();
  dialog.resize (size);

  //dialog.restoreState (settings->value("file_dialog").toByteArray());

  dialog.setFilter(QDir::AllEntries | QDir::Hidden);

  QList<QUrl> sidebarUrls = dialog.sidebarUrls();
  QList<QUrl> sidebarUrls_old = dialog.sidebarUrls();

  sidebarUrls.append(QUrl::fromLocalFile(dir_templates));
  sidebarUrls.append(QUrl::fromLocalFile(dir_snippets));
  sidebarUrls.append(QUrl::fromLocalFile(dir_sessions));
  sidebarUrls.append(QUrl::fromLocalFile(dir_scripts));

#ifdef Q_WS_X11

  QDir volDir("/mnt");
  QStringList volumes (volDir.entryList(volDir.filter() | QDir::NoDotAndDotDot));

  for (int i = 0; i < volumes.count(); i++)
      {
       sidebarUrls.append(QUrl::fromLocalFile("/mnt/" + volumes[i]));
      }

#endif

  dialog.setSidebarUrls(sidebarUrls);

  dialog.setFileMode (QFileDialog::ExistingFiles);
  dialog.setAcceptMode (QFileDialog::AcceptOpen);


  CDocument *d = documents->get_current();
  if (d)
     {
      if (file_exists (d->file_name))
          dialog.setDirectory (get_file_path (d->file_name));
      else
          dialog.setDirectory (dir_last);
     }
  else
      dialog.setDirectory (dir_last);


  dialog.setFilter (tr ("All (*);;Text files (*.txt);;Markup files (*.xml *.html *.htm *.xhtml);;C/C++ (*.c *.h *.cpp *.hh *.c++ *.h++)"));

  QLabel *l = new QLabel (tr("Charset"));
  QComboBox *cb_codecs = new QComboBox (&dialog);
  dialog.layout()->addWidget(l);
  dialog.layout()->addWidget(cb_codecs);

  if (sl_last_used_charsets.size () > 0)
     cb_codecs->addItems (sl_last_used_charsets + sl_charsets);
  else
     {
      cb_codecs->addItems (sl_charsets);
      cb_codecs->setCurrentIndex (sl_charsets.indexOf ("UTF-8"));
     }

  QStringList fileNames;
  if (dialog.exec())
     {
      dialog.setSidebarUrls(sidebarUrls_old);

      fileNames = dialog.selectedFiles();
      for (int i = 0; i < fileNames.size(); ++i)
          {
           CDocument *d = documents->open_file (fileNames.at(i), cb_codecs->currentText());
           if (d)
              {
               dir_last = get_file_path (d->file_name);
               charset = d->charset;
              }

           add_to_last_used_charsets (cb_codecs->currentText());
          }
     }
  else
       dialog.setSidebarUrls(sidebarUrls_old);

  settings->setValue ("dialog_size", dialog.size());
  //settings->setValue ("file_dialog", dialog.saveState());
  update_dyn_menus();
}

#else


void rvln::open()
{
  QFileDialog dialog (this);
  QSize size = settings->value ("dialog_size", QSize(width(), height())).toSize();
  dialog.resize (size);

  //dialog->restoreState (settings->value("file_dialog").toByteArray());

  dialog.setFilter(QDir::AllEntries | QDir::Hidden);

  QList<QUrl> sidebarUrls = dialog.sidebarUrls();
  QList<QUrl> sidebarUrls_old = dialog.sidebarUrls();

  sidebarUrls.append(QUrl::fromLocalFile(dir_templates));
  sidebarUrls.append(QUrl::fromLocalFile(dir_snippets));
  sidebarUrls.append(QUrl::fromLocalFile(dir_sessions));
  sidebarUrls.append(QUrl::fromLocalFile(dir_scripts));

  dialog.setSidebarUrls(sidebarUrls);

  dialog.setFileMode (QFileDialog::ExistingFiles);
  dialog.setAcceptMode (QFileDialog::AcceptOpen);


  CDocument *d = documents->get_current();
  if (d)
     {
      if (file_exists (d->file_name))
          dialog.setDirectory (get_file_path (d->file_name));
      else
          dialog.setDirectory (dir_last);
     }
  else
      dialog.setDirectory (dir_last);


  dialog.setFilter (tr ("All (*);;Text files (*.txt);;Markup files (*.xml *.html *.htm *.xhtml);;C/C++ (*.c *.h *.cpp *.hh *.c++ *.h++)"));

  QLabel *l = new QLabel (tr("Charset"));
  QComboBox *cb_codecs = new QComboBox (&dialog);
  dialog.layout()->addWidget(l);
  dialog.layout()->addWidget(cb_codecs);

  if (sl_last_used_charsets.size () > 0)
     cb_codecs->addItems (sl_last_used_charsets + sl_charsets);
  else
     {
      cb_codecs->addItems (sl_charsets);
      cb_codecs->setCurrentIndex (sl_charsets.indexOf ("UTF-8"));
     }

  QStringList fileNames;
  if (dialog.exec())
     {
      dialog.setSidebarUrls(sidebarUrls_old);

      fileNames = dialog.selectedFiles();
      for (int i = 0; i < fileNames.size(); ++i)
          {
           CDocument *d = documents->open_file (fileNames.at(i), cb_codecs->currentText());
           if (d)
              {
               dir_last = get_file_path (d->file_name);
               charset = d->charset;
              }

           add_to_last_used_charsets (cb_codecs->currentText());
          }
     }
  else
       dialog.setSidebarUrls(sidebarUrls_old);

  settings->setValue ("dialog_size", dialog.size());
  //settings->setValue ("file_dialog", dialog.saveState());
  update_dyn_menus();
}

#endif


bool rvln::save()
{
  CDocument *d = documents->get_current();
  if (! d)
     return false;

  if (d->textEdit->isReadOnly())
     {
      log->log (tr("This file is opened in the read-only mode. You can save it with another name using <b>Save as</b>"), "darkmood02.mng");
      return false;
     }

  if (file_exists (d->file_name))
     d->save_with_name (d->file_name, d->charset);
  else
      return saveAs();

  if (d->file_name == fname_bookmarks)
     update_bookmarks();

  if (d->file_name == fname_programs)
     update_programs();

  return true;
}


#ifndef Q_WS_WIN

bool rvln::saveAs()
{
  CDocument *d = documents->get_current();
  if (! d)
     return false;

  if (! settings->value("use_trad_dialogs", "0").toBool())
    {
     main_tab_widget->setCurrentIndex (1);
     if (file_exists (d->file_name))
        fman_nav (get_file_path (d->file_name));
         else
             fman_nav (dir_last);

     return true;
    }

  QFileDialog dialog (this);
  QSize size = settings->value ("dialog_size", QSize(width(), height())).toSize();
  dialog.resize (size);
  //dialog.restoreState (settings->value("file_dialog").toByteArray());

  dialog.setFilter(QDir::AllEntries | QDir::Hidden);

  QList<QUrl> sidebarUrls = dialog.sidebarUrls();
  QList<QUrl> sidebarUrls_old = dialog.sidebarUrls();

  sidebarUrls.append(QUrl::fromLocalFile(dir_templates));
  sidebarUrls.append(QUrl::fromLocalFile(dir_snippets));
  sidebarUrls.append(QUrl::fromLocalFile(dir_sessions));
  sidebarUrls.append(QUrl::fromLocalFile(dir_scripts));

#ifdef Q_WS_X11

  QDir volDir("/mnt");
  QStringList volumes (volDir.entryList(volDir.filter() | QDir::NoDotAndDotDot));

  for (int i = 0; i < volumes.count(); i++)
      {
       sidebarUrls.append(QUrl::fromLocalFile("/mnt/" + volumes[i]));
      }

#endif

  dialog.setSidebarUrls(sidebarUrls);

  dialog.setFileMode (QFileDialog::AnyFile);
  dialog.setAcceptMode (QFileDialog::AcceptSave);
  dialog.setConfirmOverwrite (false);
  dialog.setDirectory (dir_last);


  QLabel *l = new QLabel (tr("Charset"));
  QComboBox *cb_codecs = new QComboBox (&dialog);
  dialog.layout()->addWidget(l);
  dialog.layout()->addWidget(cb_codecs);

  if (sl_last_used_charsets.size () > 0)
     cb_codecs->addItems (sl_last_used_charsets + sl_charsets);
  else
     {
      cb_codecs->addItems (sl_charsets);
      cb_codecs->setCurrentIndex (sl_charsets.indexOf ("UTF-8"));
     }

  if (dialog.exec())
     {
      dialog.setSidebarUrls(sidebarUrls_old);

      QString fileName = dialog.selectedFiles().at(0);

      if (file_exists (fileName))
         {
          int ret = QMessageBox::warning (this, "TEA",
                                          tr("%1 already exists\n"
                                          "Do you want to overwrite?")
                                           .arg(fileName),
                                          QMessageBox::Yes | QMessageBox::Default,
                                          QMessageBox::Cancel | QMessageBox::Escape);

          if (ret == QMessageBox::Cancel)
             return false;
         }

      d->save_with_name (fileName, cb_codecs->currentText());
      d->set_markup_mode();

      add_to_last_used_charsets (cb_codecs->currentText());

      d->set_hl ();
      QFileInfo f (d->file_name);
      dir_last = f.path();

      update_dyn_menus();
     }
   else
       dialog.setSidebarUrls(sidebarUrls_old);

  settings->setValue ("dialog_size", dialog.size());
  //settings->setValue ("file_dialog", dialog.saveState());
  return true;
}

#else

bool rvln::saveAs()
{
  CDocument *d = documents->get_current();
  if (! d)
     return false;

  QFileDialog dialog (this);
  QSize size = settings->value ("dialog_size", QSize(width(), height())).toSize();
  dialog.resize (size);
  //dialog->restoreState (settings->value("file_dialog").toByteArray());

  dialog.setFilter(QDir::AllEntries | QDir::Hidden);

  QList<QUrl> sidebarUrls = dialog.sidebarUrls();
  QList<QUrl> sidebarUrls_old = dialog.sidebarUrls();

  sidebarUrls.append(QUrl::fromLocalFile(dir_templates));
  sidebarUrls.append(QUrl::fromLocalFile(dir_snippets));
  sidebarUrls.append(QUrl::fromLocalFile(dir_sessions));
  sidebarUrls.append(QUrl::fromLocalFile(dir_scripts));


  dialog.setSidebarUrls(sidebarUrls);

  dialog.setFileMode (QFileDialog::AnyFile);
  dialog.setAcceptMode (QFileDialog::AcceptSave);
  dialog.setConfirmOverwrite (false);
  dialog.setDirectory (dir_last);


  QLabel *l = new QLabel (tr("Charset"));
  QComboBox *cb_codecs = new QComboBox (&dialog);
  dialog.layout()->addWidget(l);
  dialog.layout()->addWidget(cb_codecs);

  if (sl_last_used_charsets.size () > 0)
     cb_codecs->addItems (sl_last_used_charsets + sl_charsets);
  else
     {
      cb_codecs->addItems (sl_charsets);
      cb_codecs->setCurrentIndex (sl_charsets.indexOf ("UTF-8"));
     }

  if (dialog.exec())
     {
      dialog.setSidebarUrls(sidebarUrls_old);

      QString fileName = dialog.selectedFiles().at(0);

      if (file_exists (fileName))
         {
          int ret = QMessageBox::warning (this, "TEA",
                                          tr("%1 already exists\n"
                                          "Do you want to overwrite?")
                                           .arg(fileName),
                                          QMessageBox::Yes | QMessageBox::Default,
                                          QMessageBox::Cancel | QMessageBox::Escape);

          if (ret == QMessageBox::Cancel)
             return false;
         }

      d->save_with_name (fileName, cb_codecs->currentText());
      d->set_markup_mode();

      add_to_last_used_charsets (cb_codecs->currentText());

      d->set_hl ();
      QFileInfo f (d->file_name);
      dir_last = f.path();

      update_dyn_menus();
     }
   else
       dialog.setSidebarUrls(sidebarUrls_old);

  settings->setValue ("dialog_size", dialog.size());
  //settings->setValue ("file_dialog", dialog.saveState());
  return true;
}



#endif


void rvln::about()
{
  CAboutWindow *a = new CAboutWindow();
  a->move (x() + 20, y() + 20);
  a->show();
}


void rvln::createActions()
{
  act_test = new QAction(QIcon(":/icons/file-save.svg"), tr("Test"), this);
  act_test->setShortcut(QKeySequence("Ctrl+T"));
  connect(act_test, SIGNAL(triggered()), this, SLOT(test()));

  newAct = new QAction(QIcon(":/icons/file-new.svg"), tr("New"), this);
  newAct->setShortcut(QKeySequence("Ctrl+N"));
  newAct->setStatusTip(tr("Create a new file"));
  connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

  openAct = new QAction(QIcon(":/icons/file-open.svg"), tr("Open"), this);
  openAct->setStatusTip(tr("Open an existing file"));
  connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

  saveAct = new QAction(QIcon(":/icons/file-save.svg"), tr("Save"), this);
  saveAct->setShortcut(QKeySequence("Ctrl+S"));
  saveAct->setStatusTip(tr("Save the document to disk"));
  connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

  saveAsAct = new QAction(QIcon(":/icons/file-save-as.svg"), tr("Save As"), this);
  saveAsAct->setStatusTip(tr("Save the document under a new name"));
  connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

  exitAct = new QAction(tr("Exit"), this);
  exitAct->setShortcut(QKeySequence("Ctrl+Q"));
  exitAct->setStatusTip(tr("Exit the application"));
  connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

  cutAct = new QAction(QIcon(":/icons/edit-cut.svg"), tr("Cut"), this);
  cutAct->setShortcut(QKeySequence("Ctrl+X"));
  cutAct->setStatusTip(tr("Cut the current selection's contents to the clipboard"));
  connect(cutAct, SIGNAL(triggered()), this, SLOT(ed_cut()));

  copyAct = new QAction(QIcon(":/icons/edit-copy.svg"), tr("Copy"), this);
  copyAct->setShortcut(QKeySequence("Ctrl+C"));
  copyAct->setStatusTip(tr("Copy the current selection's contents to the clipboard"));
  connect(copyAct, SIGNAL(triggered()), this, SLOT(ed_copy()));

  pasteAct = new QAction(QIcon(":/icons/edit-paste.svg"), tr("Paste"), this);
  pasteAct->setShortcut(QKeySequence("Ctrl+V"));
  pasteAct->setStatusTip(tr("Paste the clipboard's contents into the current selection"));
  connect(pasteAct, SIGNAL(triggered()), this, SLOT(ed_paste()));

  undoAct = new QAction(QIcon(":/icons/back.svg"), tr("Undo"), this);
  undoAct->setShortcut(QKeySequence("Ctrl+Z"));
  connect(undoAct, SIGNAL(triggered()), this, SLOT(ed_undo()));

  redoAct = new QAction(QIcon(":/icons/forw.svg"), tr("Redo"), this);
  connect(redoAct, SIGNAL(triggered()), this, SLOT(ed_redo()));

  aboutAct = new QAction(tr("About"), this);
  connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

  aboutQtAct = new QAction(tr("About Qt"), this);
  connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}


void rvln::createMenus()
{
  fileMenu = menuBar()->addMenu(tr("File"));
  fileMenu->setTearOffEnabled (TRUE);

  //fileMenu->addAction(act_test);

  fileMenu->addAction(newAct);
  add_to_menu (fileMenu, tr("Open"), SLOT(open()), "Ctrl+O", ":/icons/file-open.svg");
  add_to_menu (fileMenu, tr("Last opened file"), SLOT(file_last_opened()));
  add_to_menu (fileMenu, tr("Open at cursor"), SLOT(open_at_cursor()), "F2");
  add_to_menu (fileMenu, tr("Crapbook"), SLOT(file_crapbook()), "Alt+M");

  fileMenu->addSeparator();

  fileMenu->addAction(saveAct);
  fileMenu->addAction(saveAsAct);

  QMenu *tm = fileMenu->addMenu (tr("Save as different"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("Save .bak"), SLOT(file_save_bak()), "Ctrl+B");
  add_to_menu (tm, tr("Save timestamped version"), SLOT(file_save_version()), "Alt+S");
  add_to_menu (tm, tr("Save session"), SLOT(session_save_as()));



  fileMenu->addSeparator();

  menu_file_recent = fileMenu->addMenu (tr("Recent files"));

  menu_file_bookmarks = fileMenu->addMenu (tr("Bookmarks"));
  add_to_menu (fileMenu, tr("Add to bookmarks"), SLOT(file_add_to_bookmarks()));

  menu_file_templates = fileMenu->addMenu (tr("Templates"));
  menu_file_sessions = fileMenu->addMenu (tr("Sessions"));

  menu_file_configs = fileMenu->addMenu (tr("Configs"));
  add_to_menu (menu_file_configs, tr("Bookmarks list"), SLOT(file_open_bookmarks_file()));
  add_to_menu (menu_file_configs, tr("Programs list"), SLOT(file_open_programs_file()));

  fileMenu->addSeparator();

  add_to_menu (fileMenu, tr("Print"), SLOT(file_print()));
  add_to_menu (fileMenu, tr("Close current"), SLOT(close_current()), "Ctrl+W");

  fileMenu->addAction(exitAct);


  editMenu = menuBar()->addMenu(tr("Edit"));
  editMenu->setTearOffEnabled (TRUE);

  editMenu->addAction(cutAct);
  editMenu->addAction(copyAct);
  editMenu->addAction(pasteAct);

  editMenu->addSeparator();

  add_to_menu (editMenu, tr("Copy current file name"), SLOT(edit_copy_current_fname()));

  editMenu->addSeparator();

  editMenu->addAction(undoAct);
  editMenu->addAction(redoAct);

// n.p. James Plotkin - Yes. Well. No.

  menu_markup = menuBar()->addMenu(tr("Markup"));
  menu_markup->setTearOffEnabled (TRUE);

  tm = menu_markup->addMenu (tr("Mode"));
  tm->setTearOffEnabled (TRUE);

  create_menu_from_list (this, tm,
                         QString("HTML XHTML Docbook LaTeX").split (" "),
                         SLOT (mrkup_mode_choosed()));

  tm = menu_markup->addMenu (tr("Header"));
  tm->setTearOffEnabled (TRUE);

  create_menu_from_list (this, tm,
                         QString("H1 H2 H3 H4 H5 H6").split (" "),
                         SLOT (mrkup_header()));

  tm = menu_markup->addMenu (tr("Align"));
  tm->setTearOffEnabled (TRUE);

  create_menu_from_list (this, tm,
                         QString("Center Left Right Justify").split (" "),
                         SLOT (mrkup_align()));

  add_to_menu (menu_markup, tr("Bold"), SLOT(mrkup_bold()), "Alt+B");
  add_to_menu (menu_markup, tr("Italic"), SLOT(mrkup_italic()), "Alt+I");
  add_to_menu (menu_markup, tr("Link"), SLOT(mrkup_link()), "Alt+L");
  add_to_menu (menu_markup, tr("Paragraph"), SLOT(mrkup_para()), "Alt+P");
  add_to_menu (menu_markup, tr("Color"), SLOT(mrkup_color()), "Alt+C");

  add_to_menu (menu_markup, tr("Break line"), SLOT(mrkup_br()), "Ctrl+Return");
  add_to_menu (menu_markup, tr("Non-breaking space"), SLOT(mrkup_nbsp()), "Ctrl+Space");
  add_to_menu (menu_markup, tr("Insert image"), SLOT(markup_ins_image()));

  tm = menu_markup->addMenu (tr("[X]HTML tools"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("Text to [X]HTML"), SLOT(mrkup_text_to_html()));
  add_to_menu (tm, tr("Convert tags to entities"), SLOT(mrkup_tags_to_entities()));
  add_to_menu (tm, tr("Antispam e-mail"), SLOT(fn_antispam_email()));
  add_to_menu (tm, tr("Document weight"), SLOT(mrkup_document_weight()));


  menu_search = menuBar()->addMenu(tr("Search"));
  menu_search->setTearOffEnabled (TRUE);

  add_to_menu (menu_search, tr("Find"), SLOT(search_find()),"Ctrl+F");
  add_to_menu (menu_search, tr("Find next"), SLOT(search_find_next()),"F3");
  add_to_menu (menu_search, tr("Find previous"), SLOT(search_find_prev()),"Ctrl+F3");
  add_to_menu (menu_search, tr("Replace with"), SLOT(search_replace_with()));
  add_to_menu (menu_search, tr("Replace all"), SLOT(search_replace_all()));


  menu_functions = menuBar()->addMenu(tr("Functions"));
  menu_functions->setTearOffEnabled (TRUE);

  menu_fn_snippets = menu_functions->addMenu (tr("Snippets"));
  menu_fn_scripts = menu_functions->addMenu (tr("Scripts"));

  tm = menu_functions->addMenu (tr("Place"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, "Lorem ipsum", SLOT(fn_insert_loremipsum()));
  add_to_menu (tm, tr("HTML template"), SLOT(fn_insert_template_html()));
  add_to_menu (tm, tr("Date"), SLOT(fn_insert_date()));
  add_to_menu (tm, tr("Time"), SLOT(fn_insert_time()));


  tm = menu_functions->addMenu (tr("Case"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("UPCASE"), SLOT(upCase()),"Ctrl+Up");
  add_to_menu (tm, tr("lower case"), SLOT(dnCase()),"Ctrl+Down");


  tm = menu_functions->addMenu (tr("Sort"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("Sort case sensitively"), SLOT(fn_sort_casecare()));
  add_to_menu (tm, tr("Flip a list"), SLOT(fn_flip_a_list()));


  tm = menu_functions->addMenu (tr("Filter"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("Remove duplicates"), SLOT(fn_filter_rm_duplicates()));
  add_to_menu (tm, tr("Remove empty lines"), SLOT(fn_filter_rm_empty()));
  add_to_menu (tm, tr("Remove lines < N size"), SLOT(fn_filter_rm_less_than()));
  add_to_menu (tm, tr("Remove lines > N size"), SLOT(fn_filter_rm_greater_than()));
  add_to_menu (tm, tr("Filter with regexp"), SLOT(fn_filter_with_regexp()));


  tm = menu_functions->addMenu (tr("Math"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("Evaluate"), SLOT(fn_evaluate()), "F4");
  add_to_menu (tm, tr("Arabic to Roman"), SLOT(fn_number_arabic_to_roman()));
  add_to_menu (tm, tr("Roman to Arabic"), SLOT(fn_number_roman_to_arabic()));
  add_to_menu (tm, tr("Enumerate"), SLOT(fn_enum()));



  tm = menu_functions->addMenu (tr("Morse code"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("From Russian to Morse"), SLOT(fn_morse_from_ru()));
  add_to_menu (tm, tr("From Morse To Russian"), SLOT(fn_morse_to_ru()));

  add_to_menu (tm, tr("From English to Morse"), SLOT(fn_morse_from_en()));
  add_to_menu (tm, tr("From Morse To English"), SLOT(fn_morse_to_en()));

/////////////////
/*
      tm = menu_functions->addMenu (tr("Completion"));
      tm->setTearOffEnabled (TRUE);

      add_to_menu (tm, tr("Suck all words"), SLOT(fn_suck_words()), "", "");*/
////////////////

  tm = menu_functions->addMenu (tr("Statistics"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("Text statistics"), SLOT(fn_text_stat()));


  tm = menu_functions->addMenu (tr("Extraction"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("Extract words"), SLOT(fn_extract_words()));


  tm = menu_functions->addMenu (tr("Text"));
  tm->setTearOffEnabled (TRUE);

  add_to_menu (tm, tr("Apply to each line"), SLOT(fn_apply_to_each_line()),"Alt+E");
  add_to_menu (tm, tr("Remove formatting"), SLOT(fn_rm_formatting()));
  add_to_menu (tm, tr("Remove formatting at each line"), SLOT(fn_rm_formatting_at_each_line()));
  add_to_menu (tm, tr("Reverse"), SLOT(fn_reverse()));
  add_to_menu (tm, tr("Quotes to facing quotes"), SLOT(fn_convert_quotes()));



#ifdef ASPELL_ENABLE

  menu_functions->addSeparator();

  menu_spell_langs = menu_functions->addMenu (tr("Spell-checker languages"));
  menu_spell_langs->setTearOffEnabled (TRUE);

  add_to_menu (menu_functions, tr("Spell check"), SLOT(fn_spell_check()), "", ":/icons/fn-spell-check.svg");
  add_to_menu (menu_functions, tr("Suggest"), SLOT(fn_spell_suggest()));
  add_to_menu (menu_functions, tr("Add to dictionary"), SLOT(fn_spell_add_to_dict()));

  menu_functions->addSeparator();

#endif

  menu_programs = menuBar()->addMenu(tr("Run"));


  menu_nav = menuBar()->addMenu(tr("Nav"));
  menu_nav->setTearOffEnabled (TRUE);

  add_to_menu (menu_nav, tr("Save position"), SLOT(nav_save_pos()));
  add_to_menu (menu_nav, tr("Go to saved position"), SLOT(nav_goto_pos()));
  add_to_menu (menu_nav, tr("Go to line"), SLOT(nav_goto_line()),"Alt+G");
  add_to_menu (menu_nav, tr("Next tab"), SLOT(nav_goto_right_tab()));
  add_to_menu (menu_nav, tr("Prev tab"), SLOT(nav_goto_left_tab()));
  add_to_menu (menu_nav, tr("Toggle header/source"), SLOT(nav_toggle_hs()));
  add_to_menu (menu_nav, tr("Focus the Famous text entry"), SLOT(nav_focus_to_fte()));
  add_to_menu (menu_nav, tr("Focus the editor"), SLOT(nav_focus_to_editor()));


  menu_view = menuBar()->addMenu(tr("View"));
  menu_view->setTearOffEnabled (TRUE);

  add_to_menu (menu_view, tr("Toggle word wrap"), SLOT(toggle_wrap()));
  add_to_menu (menu_view, tr("Hide error marks"), SLOT(view_hide_error_marks()));
  add_to_menu (menu_view, tr("Toggle fullscreen"), SLOT(view_toggle_fs()));
  add_to_menu (menu_view, tr("Stay on top"), SLOT(view_stay_on_top()));
  add_to_menu (menu_view, tr("Preview with default browser"), SLOT(view_preview_in_bro()));


  helpMenu = menuBar()->addMenu(tr("Help"));
  helpMenu->setTearOffEnabled (TRUE);

  helpMenu->addAction(aboutAct);
  helpMenu->addAction(aboutQtAct);
  add_to_menu (helpMenu, tr ("NEWS"), SLOT(help_show_news()));
  add_to_menu (helpMenu, "TODO", SLOT(help_show_todo()));
  add_to_menu (helpMenu, "ChangeLog", SLOT(help_show_changelog()));
  add_to_menu (helpMenu, tr("License"), SLOT(help_show_gpl()));
}


void rvln::createToolBars()
{
  openAct->setMenu (menu_file_recent);

  fileToolBar = addToolBar (tr("File"));
  fileToolBar->setObjectName ("fileToolBar");
  fileToolBar->addAction (newAct);
  fileToolBar->addAction (openAct);
  fileToolBar->addAction (saveAct);

  editToolBar = addToolBar (tr("Edit"));
  editToolBar->setObjectName ("editToolBar");
  editToolBar->addAction (cutAct);
  editToolBar->addAction (copyAct);
  editToolBar->addAction (pasteAct);
}


void rvln::createStatusBar()
{
  statusBar()->showMessage (tr("Ready"));
}


rvln::~rvln()
{
//  delete inf;
  delete shortcuts;
}


void rvln::pageChanged (int index)
{
  CDocument *d = documents->list[index];
  if (! d)
     return;
/*
  if (file_exists (d->file_name))
     {
      QFileInfo f (d->file_name);
      dir_last = f.path();
     }
*/
  d->update_title ();
  d->update_status ();
}


void rvln::close_current()
{
  documents->close_current();
//  inf->change_movie ("darkmood01.mng");
}


void rvln::ed_copy()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->copy();
}


void rvln::ed_cut()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->cut();
//  inf->change_movie ("cut.mng");
}


void rvln::ed_paste()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->paste();
}


void rvln::ed_undo()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->paste();
}


void rvln::ed_redo()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->paste();
}


void rvln::ed_clear()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->paste();
}


void rvln::upCase()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (d->textEdit->textCursor().selectedText().toUpper());
}


void rvln::dnCase()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (d->textEdit->textCursor().selectedText().toLower());
}


void rvln::remove_formatting()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (d->textEdit->textCursor().selectedText().simplified());
}


QAction* rvln::add_to_menu (QMenu *menu,
                            const QString &caption,
                            const char *method,
                            const QString &shortkt,
                            const QString &iconpath
                           )
{
  QAction *act = new QAction (caption, this);

  if (! shortkt.isEmpty())
     act->setShortcut(shortkt);

  if (! iconpath.isEmpty())
     act->setIcon(QIcon (iconpath));

  connect (act, SIGNAL(triggered()), this, method);
  menu->addAction(act);
  return act;
}


void rvln::mrkup_bold()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();

  QString tag_a = "";
  QString tag_b = "";

  if (d->markup_mode == "Docbook")
     {
      tag_a = "<emphasis role=\"bold\">";
      tag_b = "</emphasis>";
     }
  else
  if (d->markup_mode == "LaTeX")
     {
      tag_a = "\\textbf{";
      tag_b = "} ";
     }
   else
       {
        tag_a = "<b>";
        tag_b = "</b>";
       }

  s = tag_a + s + tag_b;

  d->textEdit->textCursor().insertText(s);
}


void rvln::mrkup_italic()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();

  QString tag_a = "";
  QString tag_b = "";

  if (d->markup_mode == "Docbook")
     {
      tag_a = "<emphasis>";
      tag_b = "</emphasis>";
     }
  else
  if (d->markup_mode == "LaTeX")
     {
      tag_a = "\\textit{";
      tag_b = "} ";
     }
   else
       {
        tag_a = "<i>";
        tag_b = "</i>";
       }

  s = tag_a + s + tag_b;

  d->textEdit->textCursor().insertText(s);
}


void rvln::mrkup_para()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();

  QString tag_a = "";
  QString tag_b = "";

  if (d->markup_mode == "Docbook")
     {
      tag_a = "<para>";
      tag_b = "</para>";
     }
  else
      {
       tag_a = "<p>";
       tag_b = "</p>";
      }

  s = tag_a + s + tag_b;

  d->textEdit->textCursor().insertText(s);
}


void rvln::mrkup_link()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();

  QString tag_a = "";
  QString tag_b = "";

  if (d->markup_mode == "Docbook")
     {
      tag_a = "<ulink url=\"\">";
      tag_b = "</ulink>";
     }
  else
      {
       tag_a = "<a href=\"\">";
       tag_b = "</a>";
      }

  s = tag_a + s + tag_b;

  d->textEdit->textCursor().insertText(s);
}


void rvln::mrkup_br()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString br = "";

  if (d->markup_mode == "HTML")
     br = "<br>";
  else
  if (d->markup_mode == "XHTML")
     br = "<br />";
  else
  if (d->markup_mode == "LaTeX")
     br = "\\newline";

  d->textEdit->textCursor().insertText (br);
}


void rvln::mrkup_nbsp()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText("&nbsp;");
}


void rvln::search_find()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->text_to_search = fte->text();
  d->textEdit->find (d->text_to_search);
//  inf->change_movie ("watch.mng");
}


void rvln::search_find_next()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->find (d->text_to_search);
}


void rvln::search_find_prev()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->find (d->text_to_search, QTextDocument::FindBackward);
}


void rvln::file_save_bak()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString fname = d->file_name + ".bak";
  d->save_with_name_plain (fname);
  log->log (tr("%1 is saved").arg (fname), "wait01.mng");
}


#ifndef Q_WS_WIN


void rvln::markup_ins_image()
{
  main_tab_widget->setCurrentIndex (1);
}

#else

void rvln::markup_ins_image()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QFileDialog dialog (this);
  QSize size = settings->value ("dialog_size", QSize(width(), height())).toSize();
  dialog.resize (size);


  dialog.setFileMode (QFileDialog::ExistingFiles);
  dialog.setAcceptMode (QFileDialog::AcceptOpen);

  QFileInfo inf (d->file_name);
  QDir dir (inf.absolutePath());
  dialog.setDirectory (dir);

  dialog.setFilter (tr("Image Files (*.png *.jpg *.jpeg *.bmp *.gif *.mng *.tiff *.svg)"));

  QStringList fileNames;
  if (dialog.exec())
     {
      fileNames = dialog.selectedFiles();
      for (int i = 0; i < fileNames.size(); ++i)
          {
           QString fname = fileNames.at(i).toUtf8().constData();
           QImage img;
           img.load (fname);

           QString result;

           if (d->markup_mode == "HTML")
              QTextStream(&result) << "<img src=\"" << dir.relativeFilePath (fname) <<
                                     "\" border=\"0\" alt=\"\" width=\"" << img.width() << "\" height=\"" << img.height() << "\">";
           else
           if (d->markup_mode == "XHTML")
               QTextStream(&result) << "<img src=\"" << dir.relativeFilePath (fname) <<
                                     "\" border=\"0\" alt=\"\" width=\"" << img.width() << "\" height=\"" << img.height() << "\" />";
           else
           if (d->markup_mode == "Docbook")
               QTextStream(&result) << "<mediaobject><imageobject>\n<imagedata fileref=\"" << dir.relativeFilePath (fname) <<
                                       "\"/>\n</imageobject></mediaobject>";
           else
           if (d->markup_mode == "LaTeX")
               result = "\\includegraphics{"+ dir.relativeFilePath (fname) + "}";

           d->textEdit->textCursor().insertText (result);
          }
     }

  settings->setValue ("dialog_size", dialog.size());
}


#endif


void rvln::slot_lv_menuitems_currentItemChanged (QListWidgetItem *current, QListWidgetItem *previous)
{
  QAction *a = shortcuts->find_by_caption (current->text());
  ent_shtcut->setText (a->shortcut().toString());
}


void rvln::pb_assign_hotkey_clicked ()
{
  if (! lv_menuitems->currentItem())
     return;

  if (ent_shtcut->text().isEmpty())
     return;

  shortcuts->set_new_shortcut (lv_menuitems->currentItem()->text(), ent_shtcut->text());
  shortcuts->save_to_file (shortcuts->fname);
}


void rvln::pb_remove_hotkey_clicked ()
{
  if (! lv_menuitems->currentItem())
     return;

  shortcuts->set_new_shortcut (lv_menuitems->currentItem()->text(), "");
  ent_shtcut->setText ("");
  shortcuts->save_to_file (shortcuts->fname);
}


void rvln::slot_sl_icons_size_sliderMoved (int value)
{
  setIconSize (QSize (value, value));
#ifndef Q_WS_WIN
  tb_fman_dir->setIconSize (QSize (value, value));
#endif
}


void rvln::ed_date_format_editingFinished ()
{
   settings->setValue("date_format", ed_date_format->text());
}


void rvln::ed_locale_override_editingFinished ()
{
   settings->setValue("override_locale_val", ed_locale_override->text());
}


void rvln::ed_time_format_editingFinished ()
{
   settings->setValue("time_format", ed_time_format->text());
}


void rvln::createOptions()
{
  tab_options = new QTabWidget;

  main_tab_widget->addTab (tab_options, tr("setup"));

  QWidget *page_interface = new QWidget (tab_options);
  QHBoxLayout *lt_h = new QHBoxLayout;


  QLabel *l_font = new QLabel (tr("Editor font"));

  cmb_font_name = new QFontComboBox (page_interface);
  cmb_font_name->setCurrentFont (QFont (settings->value("editor_font_name", "Monospace").toString()));

  spb_font_size = new QSpinBox (page_interface);
  spb_font_size->setRange (6, 64);
  spb_font_size->setValue (settings->value("editor_font_size", "12").toInt());

  QComboBox *cmb_styles = new QComboBox (page_interface);
  cmb_styles->addItems (QStyleFactory::keys ());
  cmb_styles->addItem("NorwegianWood");

  connect(cmb_styles, SIGNAL(currentIndexChanged ( const QString & )),
          this, SLOT(slot_style_currentIndexChanged(const QString & )));

  connect(cmb_font_name, SIGNAL(currentIndexChanged ( const QString & )),
          this, SLOT(slot_editor_fontname_changed(const QString & )));

  connect(spb_font_size, SIGNAL(valueChanged (int)), this, SLOT(slot_font_size_changed (int )));


  QLabel *l_app_font = new QLabel (tr("Interface font"));

  cmb_app_font_name = new QFontComboBox (page_interface);
  cmb_app_font_name->setCurrentFont (QFont (settings->value("app_font_name", qApp->font().family()).toString()));
  spb_app_font_size = new QSpinBox (page_interface);
  spb_app_font_size->setRange (6, 64);
  QFontInfo fi = QFontInfo (qApp->font());

  spb_app_font_size->setValue (settings->value("app_font_size", fi.pointSize()).toInt());
  connect(spb_app_font_size, SIGNAL(valueChanged (int)), this, SLOT(slot_app_font_size_changed (int )));

  connect(cmb_app_font_name, SIGNAL(currentIndexChanged ( const QString & )),
          this, SLOT(slot_app_fontname_changed(const QString & )));


  QLabel *l_icons_size = new QLabel (tr("Icons size"));
  QSlider *sl_icons_size = new QSlider (Qt::Horizontal, this);
  sl_icons_size->setRange (6, 128);

  connect(sl_icons_size, SIGNAL(valueChanged (int)),
          this, SLOT(slot_sl_icons_size_sliderMoved (int)));

  sl_icons_size->setValue (settings->value("icons_size", "24").toInt());


  QLabel *l_uistyle = new QLabel (tr("UI style"));
  cmb_styles->setCurrentIndex (cmb_styles->findText (settings->value("ui_style", "Plastique").toString()));

  QCheckBox *cb_wordwrap = new QCheckBox (tr("Word wrap"), tab_options);
  cb_wordwrap->setCheckState (Qt::CheckState (settings->value("word_wrap", "2").toInt()));
  connect(cb_wordwrap, SIGNAL(stateChanged (int )), this, SLOT(cb_wordwrap_changed (int )));

  QCheckBox *cb_hl_current_line = new QCheckBox (tr("Additional highlighting"), tab_options);
  cb_hl_current_line->setCheckState (Qt::CheckState (settings->value("additional_hl", "2").toInt()));
  connect(cb_hl_current_line, SIGNAL(stateChanged (int )), this, SLOT(cb_hl_current_line_changed (int )));

  QCheckBox *cb_cursor_xy_visible = new QCheckBox (tr("Show cursor position"), tab_options);
  cb_cursor_xy_visible->setCheckState (Qt::CheckState (settings->value("cursor_xy_visible", "2").toInt()));
  connect(cb_cursor_xy_visible, SIGNAL(stateChanged (int )), this, SLOT(cb_cursor_xy_visible_changed (int )));

#ifndef Q_WS_WIN

  QCheckBox *cb_use_trad_dialogs = new QCheckBox (tr("Use traditional File Save/Open dialogs"), tab_options);
  cb_use_trad_dialogs->setCheckState (Qt::CheckState (settings->value("use_trad_dialogs", "0").toInt()));
  connect(cb_use_trad_dialogs, SIGNAL(stateChanged (int )), this, SLOT(cb_use_trad_dialogs_changed (int )));

#endif

  QCheckBox *cb_session_restore = new QCheckBox (tr("Restore the last session on start-up"), tab_options);
  cb_session_restore->setCheckState (Qt::CheckState (settings->value("session_restore", "0").toInt()));
  connect(cb_session_restore, SIGNAL(stateChanged (int )), this, SLOT(cb_session_restore (int )));


  QHBoxLayout *hb_locovr = new QHBoxLayout;

  QCheckBox *cb_override_locale = new QCheckBox (tr("Override locale"), tab_options);
  cb_override_locale->setCheckState (Qt::CheckState (settings->value ("override_locale", 0).toInt()));
  connect(cb_override_locale, SIGNAL(stateChanged (int )), this, SLOT(cb_locale_override (int )));


  ed_locale_override = new QLineEdit (this);
  ed_locale_override->setText (settings->value ("override_locale_val", "en").toString());
  connect(ed_locale_override, SIGNAL(editingFinished()), this, SLOT(ed_locale_override_editingFinished()));

  hb_locovr->addWidget (cb_override_locale);
  hb_locovr->addWidget (ed_locale_override);

/*
  QCheckBox *cb_showroxvisor = new QCheckBox (tr("Show TEAvisor"), tab_options);
  cb_showroxvisor->setCheckState (Qt::CheckState (settings->value("showroxvisor", "2").toInt()));
  connect(cb_showroxvisor, SIGNAL(stateChanged (int )), this, SLOT(cb_showroxvisor_changed (int )));
*/

  QVBoxLayout *page_interface_layout = new QVBoxLayout;
  page_interface_layout->setAlignment (Qt::AlignTop);

  lt_h->addWidget(l_font);
  lt_h->addWidget(cmb_font_name);
  lt_h->addWidget(spb_font_size);
//  lt_h->addLayout(hb_locovr);


  page_interface_layout->addLayout (lt_h);

  lt_h = new QHBoxLayout;

  lt_h->addWidget(l_app_font);
  lt_h->addWidget(cmb_app_font_name);
  lt_h->addWidget(spb_app_font_size);

  page_interface_layout->addLayout (lt_h);

  lt_h = new QHBoxLayout;

  lt_h->addWidget(l_icons_size);
  lt_h->addWidget(sl_icons_size);

  page_interface_layout->addLayout (lt_h);


  lt_h = new QHBoxLayout;

  lt_h->addWidget(l_uistyle);
  lt_h->addWidget(cmb_styles);

  page_interface_layout->addLayout (lt_h);
  page_interface_layout->addWidget (cb_wordwrap);
  page_interface_layout->addWidget (cb_hl_current_line);
  page_interface_layout->addWidget (cb_cursor_xy_visible);


#ifndef Q_WS_WIN

  page_interface_layout->addWidget (cb_use_trad_dialogs);

#endif

  page_interface_layout->addWidget (cb_session_restore);


//  page_interface_layout->addWidget (cb_showroxvisor);


/*
  QDir dir;
  dir.setPath (":/");
  dir.cd ("translations");
  QStringList filters;
  filters << "*.qm";
  dir.setNameFilters(filters);
  QStringList files = dir.entryList ();
  files.prepend ("English");

  lt_h = new QHBoxLayout;

  QComboBox *cmb_langs = new QComboBox (page_interface);
  cmb_langs->addItems (files);
  connect(cmb_langs, SIGNAL(currentIndexChanged ( const QString & )),
          this, SLOT(cmb_translation_changed(const QString & )));

  lt_h->addWidget(cmb_langs);
*/


//  page_interface_layout->addLayout (lt_h);

  page_interface_layout->addLayout (hb_locovr);


  page_interface->setLayout(page_interface_layout);
  page_interface->show();

  tab_options->addTab (page_interface, tr("Common"));


/////////////

  QWidget *page_keyboard = new QWidget (tab_options);

  lt_h = new QHBoxLayout;

  QVBoxLayout *lt_vkeys = new QVBoxLayout;
  QVBoxLayout *lt_vbuttons = new QVBoxLayout;

  lv_menuitems = new QListWidget ();
  shortcuts->captions_iterate();
  lv_menuitems->addItems (shortcuts->captions);

  lt_vkeys->addWidget(lv_menuitems);

  connect(lv_menuitems, SIGNAL(currentItemChanged (QListWidgetItem *, QListWidgetItem *)),
           this, SLOT(slot_lv_menuitems_currentItemChanged (QListWidgetItem *, QListWidgetItem *)));

  ent_shtcut = new CShortcutEntry;
  lt_vbuttons->addWidget (ent_shtcut);

  QPushButton *pb_assign_hotkey = new QPushButton (tr("Assign"), this);
  QPushButton *pb_remove_hotkey = new QPushButton (tr("Remove"), this);

  connect(pb_assign_hotkey, SIGNAL(clicked ()), this, SLOT(pb_assign_hotkey_clicked()));
  connect(pb_remove_hotkey, SIGNAL(clicked ()), this, SLOT(pb_remove_hotkey_clicked()));

  lt_vbuttons->addWidget (pb_assign_hotkey);
  lt_vbuttons->addWidget (pb_remove_hotkey, 0, Qt::AlignTop);

  lt_h->addLayout (lt_vkeys);
  lt_h->addLayout (lt_vbuttons);

  page_keyboard->setLayout (lt_h);
  page_keyboard->show();

  tab_options->addTab (page_keyboard, tr("Keyboard"));


/////////////////////////

  QWidget *page_functions = new QWidget (tab_options);
  QVBoxLayout *page_functions_layout = new QVBoxLayout;

  QLabel *l_t = new QLabel (tr ("Date format"));
  ed_date_format = new QLineEdit (this);
  ed_date_format->setText (settings->value("date_format", "dd/MM/yyyy").toString());
  connect(ed_date_format, SIGNAL(editingFinished()), this, SLOT(ed_date_format_editingFinished()));


  lt_h = new QHBoxLayout;

  lt_h->addWidget(l_t);
  lt_h->addWidget(ed_date_format);


  page_functions_layout->addLayout (lt_h);


  l_t = new QLabel (tr ("Time format"));
  ed_time_format = new QLineEdit (this);
  ed_time_format->setText (settings->value("time_format", "hh:mm:ss").toString());
  connect(ed_time_format, SIGNAL(editingFinished ()), this, SLOT(ed_time_format_editingFinished ()));

  lt_h = new QHBoxLayout;

  lt_h->addWidget(l_t);
  lt_h->addWidget(ed_time_format);

  page_functions_layout->addLayout (lt_h);

  page_functions_layout->setAlignment (Qt::AlignTop);

  page_functions->setLayout (page_functions_layout);
  page_functions->show();

  tab_options->addTab (page_functions, tr("Functions"));

/////////////////
/*
  QWidget *page_colors = new QWidget (tab_options);
  QVBoxLayout *page_colors_layout = new QVBoxLayout;

  QComboBox *cmb_ct = new QComboBox (page_colors);
  populateFilter (cmb_ct);

  page_colors_layout->addWidget (cmb_ct);

  page_colors_layout->setAlignment (Qt::AlignTop);

  page_colors->setLayout (page_colors_layout);
  page_colors->show();

  tab_options->addTab (page_colors, tr("Colors"));
*/
}

/*
void rvln::populateFilter(QComboBox *comboBox)
{
    QPixmap pix(22, 22);

    QStringList colorNames = QColor::colorNames();
    comboBox->addItem(tr("Select a filter.."));
    pix.fill(QColor("grey"));
    comboBox->addItem(pix,tr("GreyScale"));
    comboBox->addItem(tr("RGB Swapped"));
    comboBox->addItem(tr("InvertPixels color"));
    pix.fill(QColor(162,128,101));
    comboBox->addItem(pix,tr("Colorize")+QString(" color=\"sepia\""));
    comboBox->addItem(tr("Colorize")+QString(" color=\"mycolor\""));

    foreach (QString name, colorNames) {

         pix.fill(QColor(name));
        comboBox->addItem(pix,tr("Colorize")+QString(" color=\"%1\"").arg(name));
    }
}
*/


void rvln::slot_style_currentIndexChanged (const QString &text)
{
  //QApplication::setStyle (text);

  if (text == "NorwegianWood")
     QApplication::setStyle(new NorwegianWoodStyle);
  else
      QApplication::setStyle(QStyleFactory::create(text));


//  changePalette();

  settings->setValue("ui_style", text);
}


void rvln::open_at_cursor()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

 QString fname_short = d->get_filename_at_cursor ();

 if (fname_short.isEmpty())
    return;

  if (is_image (fname_short))
     {
      new image_viewer (fname_short);
      return;
     }

 if (fname_short.startsWith("#"))
    {
     fname_short.remove (0, 1);

     fname_short.prepend ("name=\"");
     d->textEdit->find (fname_short);

     return;
    }

  documents->open_file (fname_short, d->charset);
}


void rvln::toggle_wrap()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  if (d->textEdit->lineWrapMode() == QTextEdit::NoWrap)
     d->textEdit->setLineWrapMode (QTextEdit::WidgetWidth);
  else
      d->textEdit->setLineWrapMode (QTextEdit::NoWrap);
}


void rvln::cb_wordwrap_changed (int state)
{
  settings->setValue("word_wrap", state);
}


void rvln::cb_session_restore (int state)
{
  settings->setValue("session_restore", state);
}


void rvln::cb_hl_current_line_changed (int state)
{
  settings->setValue("additional_hl", state);
  documents->apply_settings();
}


void rvln::cb_cursor_xy_visible_changed (int state)
{
  settings->setValue("cursor_xy_visible", state);
  documents->apply_settings();
}

#ifndef Q_WS_WIN

void rvln::cb_use_trad_dialogs_changed (int state)
{
  settings->setValue("use_trad_dialogs", state);
}

#endif


void rvln::cb_locale_override (int state)
{
  settings->setValue("override_locale", state);
}

/*
void rvln::cb_showroxvisor_changed (int state)
{
  settings->setValue("showroxvisor", state);
//  inf->setVisible (state);
}
*/

void rvln::view_preview_in_bro()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString cm;
  cm = "file:///" + d->file_name;
  QDesktopServices::openUrl(cm);
}


void rvln::nav_save_pos()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->position = d->textEdit->textCursor().position();
}


void rvln::nav_goto_pos()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QTextCursor cr = d->textEdit->textCursor();
  cr.setPosition (d->position);
  d->textEdit->setTextCursor (cr);
}


void rvln::slot_editor_fontname_changed (const QString &text)
{
  settings->setValue("editor_font_name", text);
  updateFonts();
}


void rvln::slot_app_fontname_changed (const QString &text)
{
  settings->setValue("app_font_name", text);
  updateFonts();
}


void rvln::slot_app_font_size_changed (int i)
{
  settings->setValue("app_font_size", i);
  updateFonts();
}


void rvln::slot_font_size_changed (int i)
{
  settings->setValue("editor_font_size", i);
  updateFonts();
}


void rvln::mrkup_color()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QColor color = QColorDialog::getColor(Qt::green, this);
  if (! color.isValid())
     return;

  QString s;

  if (d->textEdit->textCursor().hasSelection ())
      s = QString("<span style=\"color:%1;\">%2</span>")
                  .arg (color.name())
                  .arg (d->textEdit->textCursor().selectedText());
  else
      s = color.name();

  d->textEdit->textCursor().insertText(s);
}


void rvln::nav_goto_line()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QTextCursor cr = d->textEdit->textCursor();
  cr.movePosition (QTextCursor::Start);
  cr.movePosition (QTextCursor::Down, QTextCursor::MoveAnchor, fte->text().toInt());
  d->textEdit->setTextCursor (cr);
}


void rvln::updateFonts()
{
  documents->apply_settings();

  QFont f;
  f.fromString (settings->value("editor_font_name", "Monospace").toString());
  f.setPointSize (settings->value("editor_font_size", "12").toInt());
  man->setFont(f);

  QFont fapp;
  QFontInfo fi = QFontInfo (qApp->font());

  fapp.setPointSize (settings->value("app_font_size", fi.pointSize()).toInt());
  fapp.fromString (settings->value("app_font_name", qApp->font().family()).toString());
  qApp->setFont (fapp);
}


void rvln::createManual()
{
  QWidget *wd_man = new QWidget (this);

  QVBoxLayout *lv_t = new QVBoxLayout ();

  QString loc = QLocale::system().name().left(2);

  if (settings->value ("override_locale", 0).toBool())
     {
      QString ts = settings->value ("override_locale_val", "en").toString();
      if (ts.length() != 2)
          ts = "en";
      loc = ts;
     }

  QString filename = ":/manuals/" + loc + ".html";
  if (! file_exists (filename))
      filename = ":/manuals/en.html";

  man = new QTextBrowser();
  man->setOpenExternalLinks (TRUE);
  man->setSource (QUrl("qrc" + filename));
  QToolBar *b_nav =  new QToolBar();
  b_nav->setIconSize (QSize(12, 12));

  QAction *act_back = b_nav->addAction (QIcon (":/icons/back.svg"), tr("Back"));
  QAction *act_forw = b_nav->addAction (QIcon (":/icons/forw.svg"), tr("Next"));

  connect(act_back, SIGNAL(triggered()), man, SLOT(backward()));
  connect(act_forw, SIGNAL(triggered()), man, SLOT(forward()));

  lv_t->addWidget (b_nav);
  lv_t->addWidget (man);

  wd_man->setLayout (lv_t);

  main_tab_widget->addTab (wd_man, tr("learn"));
}


void rvln::file_crapbook()
{
  QFile file (fname_crapbook);
  if (! file.exists())
     {
      QFile file (fname_crapbook);
      if (file.open (QFile::WriteOnly | QFile::Text))
         {
          QTextStream out (&file);
          out.setCodec ("UTF-8");
          out << "some text";
         }
       file.close();
     }

  documents->open_file (fname_crapbook, "UTF-8");
}


void rvln::fn_apply_to_each_line()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();

  QStringList sl = s.split (QChar::ParagraphSeparator);

  for (int i = 0; i < sl.size(); ++i)
      sl[i] = QString(fte->text()).arg(sl[i]);

  d->textEdit->textCursor().insertText (sl.join ("\n"));
}


void rvln::fn_filter_with_regexp()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();
  QStringList sl = s.split (QChar::ParagraphSeparator);
  QStringList result = sl.filter (QRegExp (fte->text()));
  d->textEdit->textCursor().insertText (result.join ("\n"));
}


void rvln::fn_reverse()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();
  if (s.isEmpty())
     return;

  QString sn;

  int c = s.length() - 1;
  int x = 0;

  for (int i = c; i >= 0; i--)
      sn[x++] = s[i];

  d->textEdit->textCursor().insertText (sn);
}


void rvln::file_print()
{

  CDocument *d = documents->get_current();
  if (! d)
     return;

  QPrintDialog *dialog = new QPrintDialog(&printer, this);

  dialog->setWindowTitle(tr("Print Document"));

  if (d->textEdit->textCursor().hasSelection())
       dialog->addEnabledOption(QAbstractPrintDialog::PrintSelection);

  if (dialog->exec() != QDialog::Accepted)
      return;

  d->textEdit->print (&printer);
}


void rvln::file_last_opened()
{
  if (documents->recent_files.size() > 0)
     {
      documents->open_file_triplex (documents->recent_files[0]);
      documents->recent_files.removeAt (0);
      documents->update_recent_menu ();
     }
}


#ifdef ASPELL_ENABLE

void rvln::fn_change_spell_lang()
{
  QAction *Act = qobject_cast<QAction *>(sender());
  settings->setValue("spell_lang", Act->text());
  spellchecker->change_lang (Act->text ());
  fn_spell_check();
}


void rvln::create_spellcheck_menu()
{
  menu_spell_langs->clear ();
  create_menu_from_list (this, menu_spell_langs, spellchecker->get_speller_modules_list(), SLOT(fn_change_spell_lang()));
}


bool char_is_shit (const QChar &c)
{
  if (c == '(' || c == '"' || c == ' ' || c == '-')
     return true;

  return false;
}


void rvln::fn_spell_check()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  //pb_status->show();
  //pb_status->setRange (0, 0);

  //timer.start();

//  inf->change_movie ("wait03.mng");

  QTextCursor cr = d->textEdit->textCursor();
  int pos = cr.position();

  QString text = d->textEdit->toPlainText();
//delete all underlines
  cr.setPosition (0);
  cr.movePosition (QTextCursor::End, QTextCursor::KeepAnchor);
  QTextCharFormat f = cr.blockCharFormat ();
  f.setFontUnderline (false);
  cr.mergeCharFormat (f);

  cr.setPosition (0);
  cr.movePosition (QTextCursor::Start, QTextCursor::MoveAnchor);

  while (cr.movePosition (QTextCursor::NextWord))
        {

//fix here all that stuff
         QChar c = text[cr.position()];
         if (! c.isNull() && !c.isLetter())
             while (! c.isNull() && ! c.isLetter())
                  {
                   cr.movePosition (QTextCursor::NextCharacter);
                   c = text[cr.position()];
                  }

         //a dirty hack:

//         if (c == '(' || c == '"' || c == ' ' || c == '-')
  //          cr.movePosition (QTextCursor::NextCharacter);

         //if (c == '(' || c == '"' || c == ' ' || c == '-')
/*         if (char_is_shit(c))
            {
             while (cr.movePosition (QTextCursor::NextCharacter))
                  {
                   if (! c.isSpace())
                       break;
                   //if (! c.isPunct())
                     //  break;

                  }
            }
*/

/*         cr.movePosition (QTextCursor::EndOfWord, QTextCursor::KeepAnchor);

         c = text[cr.position()];
         if (c.isPunct())
            cr.movePosition (QTextCursor::PreviousCharacter);
*/

        cr.movePosition (QTextCursor::EndOfWord, QTextCursor::KeepAnchor);

        c = text[cr.position()];
        if (c.isNull() && ! c.isLetter())
            while (! c.isNull() && !c.isLetter())
                  {
                   cr.movePosition (QTextCursor::PreviousCharacter);
                   c = text[cr.position()];
                  }


         QString stext = cr.selectedText();
         if (! stext.isNull() || ! stext.isEmpty())
         if (! spellchecker->check (cr.selectedText()))
            {
             qDebug() << "|" << cr.selectedText() << "|";

             f = cr.blockCharFormat ();
             f.setUnderlineStyle (QTextCharFormat::WaveUnderline);
             f.setUnderlineColor (Qt::red);
             f.setFontUnderline (true);
             cr.mergeCharFormat (f);
            }


         //pb_status->setValue (i++);
    //     if (pbar_step % 100 == 0)
      //      qApp->processEvents();
        }

   cr.setPosition (pos);
   d->textEdit->setTextCursor (cr);
   d->textEdit->document()->setModified (false);

   //timer.stop();
   //pb_status->hide();
}


void rvln::fn_spell_add_to_dict()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QTextCursor cr = d->textEdit->textCursor();
  cr.select (QTextCursor::WordUnderCursor);
  QString s = cr.selectedText();

  if (! s.isEmpty())
     spellchecker->add_to_user_dict (s);
}


void rvln::fn_spell_suggest_callback()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QAction *Act = qobject_cast<QAction *>(sender());
  QString new_text = Act->text();

  QTextCursor cr = d->textEdit->textCursor();

  cr.select (QTextCursor::WordUnderCursor);
  QString s = cr.selectedText();
  if (s[0].isUpper())
     new_text[0] = new_text[0].toUpper();

  cr.insertText (new_text);
  d->textEdit->setTextCursor (cr);
}


void rvln::fn_spell_suggest()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QTextCursor cr = d->textEdit->textCursor();
  cr.select (QTextCursor::WordUnderCursor);
  QString s = cr.selectedText();
  if (s.isEmpty())
     return;

  QStringList l = spellchecker->get_suggestions_list (s);

  QMenu *m = new QMenu (this);
  create_menu_from_list (this, m, l, SLOT (fn_spell_suggest_callback()));
  m->popup (mapToGlobal(d->textEdit->cursorRect().topLeft()));
}

#endif


CShortcuts::CShortcuts (QWidget *widget)
{
  w = widget;
}


void CShortcuts::captions_iterate ()
{
  captions.clear();
  QList<QAction *> a = w->findChildren<QAction *>();

  int c = a.size();

  for (int i = 0; i < c; i++)
      if (! a[i]->text().isEmpty())
         captions.prepend (a[i]->text());

  captions.sort ();
}


QAction* CShortcuts::find_by_caption (const QString &text)
{
  QList<QAction *> a = w->findChildren<QAction *>();

  int c = a.size();

  for (int i = 0; i < c; i++)
      if (a[i]->text() == text)
         return a[i];

  return NULL;
}


QAction* CShortcuts::find_by_shortcut (const QString &shcut)
{
  QList<QAction *> a = w->findChildren<QAction *>();

  int c = a.size();

  for (int i = 0; i < c; i++)
      if (a[i]->shortcut().toString() == shcut)
         return a[i];

  return NULL;
}


QKeySequence CShortcuts::find_seq_by_caption (const QString &text)
{
  QAction *a = find_by_caption (text);

  if (a)
     return a->shortcut();

   return QKeySequence::fromString ("Ctrl+Alt+Z");
}


void CShortcuts::set_new_shortcut (const QString &menuitem, const QString &shcut)
{
  QAction *b = find_by_shortcut (shcut);
  if (b)
      b->setShortcut (QKeySequence(""));

  QAction *a = find_by_caption (menuitem);
  if (a)
     a->setShortcut (QKeySequence (shcut));
}


void CShortcuts::save_to_file (const QString &file_name)
{
  QList<QAction *> a = w->findChildren<QAction *>();

  int c = a.size();

  QString s;

  for (int i = 0; i < c; i++)
      {
       if (! a[i]->shortcut().toString().isEmpty())
          s += a[i]->text() + "=" + a[i]->shortcut().toString() + "\n";
      }

  qstring_save (file_name, s);
}


void CShortcuts::load_from_file (const QString &file_name)
{
  if (! file_exists (file_name))
     return;

  QHash <QString, QString> hash;
  QString s = qstring_load (file_name);
  QStringList l = s.split ("\n");
  int c = l.size();

  for (int x = 0; x < c; x++)
     {
      QStringList pair = l[x].split ("=");
      if (pair.size() == 2)
         hash.insert (pair[0], pair[1]);
     }


  QList<QAction *> a = w->findChildren<QAction *>();

  int z = a.size();
  for (int i = 0; i < z; i++)
      {
       if (hash.contains (a[i]->text()))
          a[i]->setShortcut (QKeySequence(hash.value(a[i]->text())));
      }
}


void CShortcutEntry::keyPressEvent (QKeyEvent *event)
{
  event->accept ();
  setText (mod_to_string (event->modifiers()) + keycode_to_string (event->key()));
}


void rvln::file_open_bookmark()
{
  QAction *a = qobject_cast<QAction *>(sender());
  documents->open_file_triplex (a->text());
}


void rvln::file_add_to_bookmarks()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  if (! file_exists (d->file_name))
     return;

  bool found = false;
  QStringList l_bookmarks = qstring_load (fname_bookmarks).split("\n");

  for (int i = 0; i < l_bookmarks.size(); i++)
      {
       if (l_bookmarks[i].contains (d->file_name))
          {
           l_bookmarks[i] = d->get_triplex();
           found = true;
           break;
          }
      }

  if (! found)
     l_bookmarks.prepend (d->get_triplex());

  bookmarks = l_bookmarks.join ("\n").trimmed();

  qstring_save (fname_bookmarks, bookmarks);
  update_bookmarks ();
}


void rvln::file_use_template()
{
  QAction *a = qobject_cast<QAction *>(sender());

  CDocument *d = documents->create_new();
  d->textEdit->textCursor().insertText (qstring_load (dir_templates + "/" + a->text()));
}


void rvln::file_use_snippet()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QAction *a = qobject_cast<QAction *>(sender());
  QString s = qstring_load (dir_snippets + "/" + a->text());

  if (s.contains ("%s"))
     s = s.replace ("%s", d->textEdit->textCursor().selectedText());

  d->textEdit->textCursor().insertText (s);
}


void rvln::update_templates()
{
  menu_file_templates->clear ();
  QStringList l = read_dir_entries (dir_templates);

  create_menu_from_list (this, menu_file_templates,
                         l,
                         SLOT (file_use_template()));
}


void rvln::update_snippets()
{
  menu_fn_snippets->clear ();
  QStringList l = read_dir_entries (dir_snippets);

  create_menu_from_list (this, menu_fn_snippets,
                         l,
                         SLOT (file_use_snippet()));
}


void rvln::file_save_version()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  if (! file_exists (d->file_name))
     return;

  QDate date = QDate::currentDate ();
  QFileInfo fi;
  fi.setFile (d->file_name);

  QString version_timestamp_fmt = settings->value("version_timestamp_fmt", "yyyy-MM-dd").toString();
  QTime t = QTime::currentTime();

  QString fname = fi.absoluteDir().absolutePath() + "/" + date.toString (version_timestamp_fmt) + "-" + t.toString("hh-mm-ss") +
                  "-" + fi.baseName() + "." + fi.suffix();

  if (d->save_with_name_plain (fname))
     log->log (tr("%1 - saved").arg (fname), "big01.mng");
  else
     log->log (tr("Cannot save %1").arg (fname), "scary01.mng");
}


void rvln::dragEnterEvent(QDragEnterEvent *event)
{
  if (event->mimeData()->hasFormat("text/uri-list"))
     event->acceptProposedAction();
}


void rvln::dropEvent(QDropEvent *event)
{
  QList <QUrl> l;
  QString fName;
  QFileInfo info;

  if (event->mimeData()->hasUrls())
     {
      l = event->mimeData()->urls();
      if (l.size() > 0)
         {
          int c = l.size();
          for (int i = 0; i < c; i++)
              {
               fName = l[i].toLocalFile();
               info.setFile(fName);
               if (info.isFile())
                  documents->open_file (fName, "UTF-8");
              }
         }
      }

  event->acceptProposedAction();
}


void rvln::fn_evaluate()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();

  QScriptEngine e;
  QScriptValue v = e.evaluate(s);
  log->log (v.toString ());
}


void rvln::fn_sort_casecare()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();

  QStringList l = s.split (QChar::ParagraphSeparator);
  l.sort();

  d->textEdit->textCursor().insertText (l.join("\n"));
}


void rvln::mrkup_text_to_html()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();

  QStringList l = s.split (QChar::ParagraphSeparator);
  QString result;

  if (d->markup_mode == "HTML")
     result += "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd\">\n";
  else
     result += "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";

  result +=  "<html>\n"
             "<head>\n"
             "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n"
             "<style type=\"text/css\">\n"
             ".p1\n"
             "{\n"
             "margin: 0px 0px 0px 0px;\n"
             "padding: 0px 0px 0px 0px;\n"
             "text-indent: 1.5em;\n"
             "text-align: justify;\n"
             "}\n"
             "</style>\n"
             "<title></title>\n"
             "</head>\n"
             "<body>\n";

  int c = l.size () ;
  for (int i = 0; i < c; ++i)
     {
      result += "<p class=\"p1\">" + l[i] + "</p>\n";
     }

  result = result.replace ("<p class=\"p1\"></p>", "<br>");

  result += "</body>\n</html>";

  CDocument *doc = documents->create_new();
  if (doc)
     doc->textEdit->textCursor().insertText (result);
}


void rvln::fn_text_stat()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s;

  if (d->textEdit->textCursor().hasSelection())
     s = d->textEdit->textCursor().selectedText();
  else
      s = d->textEdit->toPlainText();

  int c = s.length();
  int purechars = 0;
  int lines = 1;

  for (int i = 0; i < c; ++ i)
      {
       QChar ch = s.at(i);

       if (ch.isLetterOrNumber() || ch.isPunct())
          purechars++;

       if (ch == '\n')
          lines++;
      }


  QString result = tr("chars: %1<br>chars without spaces: %2<br>lines: %3<br>author's sheets: %4")
                       .arg (QString::number (c))
                       .arg (QString::number (purechars))
                       .arg (QString::number (lines))
                       .arg (QString::number (c / 40000));

  documents->log->log (result, "wait03.mng");
}


void rvln::handle_commandline()
{
#ifndef Q_WS_WIN
/*
  QStringList l = qApp->arguments();
  int size = l.size();
  if (size < 2)
     return;

  for (int i = 1; i < size; ++i)
      {
       QFileInfo f (l[i]);

       if (! f.isAbsolute())
          {
           QString fullname = QDir::currentPath() + "/" + l[i];
           documents->open_file (fullname, "UTF-8");
          }
       else
       documents->open_file (l[i], "UTF-8");
      }
*/
#endif
}


void rvln::fn_antispam_email()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();
  QString result;

  int c = s.size();
  for (int i = 0; i < c; i++)
      result += "&#" + QString::number(s[i].unicode()) +";";

  d->textEdit->textCursor().insertText (result);
}


void rvln::search_replace_with()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (fte->text());
}


void rvln::search_replace_all()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QStringList l = fte->text().split ("~");
  if (l.size() < 2)
     return;

  QString s = d->textEdit->toPlainText().replace (l[0], l[1]);

  d->textEdit->textCursor().insertText (s);
}


void CApplication::saveState ( QSessionManager & manager )
{
   manager.setRestartHint (QSessionManager::RestartIfRunning);
}

/*
void CApplication::commitData ( QSessionManager & manager )
{
   qDebug() << "CApplication::commitData";
}
*/
/*
void rvln::cmb_translation_changed (const QString &s)
{
   settings->setValue("ui_lang", s);

   qDebug () << s;

   QTranslator t;

   if (s != "English")
       t.load(":/translations/rlvn_" + s);
   qApp->installTranslator(&t);
}
*/


void rvln::update_charsets()
{
  QString fname = dir_config + "/last_used_charsets";
  if (! file_exists (fname))
     qstring_save (fname, "UTF-8");

  sl_last_used_charsets = qstring_load (fname).split ("\n");

  QList<QByteArray> availableCodecs = QTextCodec::availableCodecs();

  for (int i = 0; i < availableCodecs.size(); ++i )
      sl_charsets.prepend (availableCodecs[i]);

  sl_charsets.sort();
}


void rvln::add_to_last_used_charsets (const QString &s)
{
  int i = sl_last_used_charsets.indexOf (s);
  if (i == -1)
     sl_last_used_charsets.prepend (s);
  else
      sl_last_used_charsets.move (i, 0);

  if (sl_last_used_charsets.size() > 3)
     sl_last_used_charsets.removeLast();
}


void rvln::fn_flip_a_list()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QStringList sl = d->textEdit->textCursor().selectedText().split (QChar::ParagraphSeparator);

  QString result;

  for (int i = sl.size() - 1; i > -1; i--)
     result += sl[i] + "\n";

  d->textEdit->textCursor().insertText (result);
}


void rvln::mrkup_tags_to_entities()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (Qt::escape (d->textEdit->textCursor().selectedText()));
}


void rvln::fn_insert_loremipsum()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (qstring_load(":/text-data/lorem-ipsum"));
}


void rvln::mrkup_mode_choosed()
{
  QAction *a = qobject_cast<QAction *>(sender());
  markup_mode = a->text();
  documents->markup_mode = markup_mode;

  CDocument *d = documents->get_current();
  if (d)
     d->markup_mode = markup_mode;

//  inf->change_movie ("smile01.mng");
}


void rvln::mrkup_header()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();
  QAction *a = qobject_cast<QAction *>(sender());
  QString h = a->text ().toLower();

  s.prepend ("<" + h + ">");
  s.append ("</" + h + ">");

  d->textEdit->textCursor().insertText (s);
}


void rvln::mrkup_align()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText();
  QAction *a = qobject_cast<QAction *>(sender());
  QString h = a->text().toLower();

  QString tag_a = "";
  QString tag_b = "";

  if (d->markup_mode == "LaTeX")
     {
      s = "\\begin{"+ h + "}\n" + s + "\\end{"+ h +"}";
     }
   else
       {
        tag_a = "<p style=\"text-align:" + h + ";\">";
        tag_b = "</p>";
        s = tag_a + s + tag_b;
       }

  d->textEdit->textCursor().insertText (s);
}


void rvln::mrkup_document_weight()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString result;
  QStringList l = html_get_by_patt (d->textEdit->toPlainText(), "src=\"");

  QFileInfo f (d->file_name);
  QUrl baseUrl(d->file_name);

  //result += d->file_name + " " + QString::number (f.size() / 1024) + " " + tr("kbytes<br>");
  result += tr ("%1 %2 kbytes<br>").arg(d->file_name).arg(QString::number (f.size() / 1024));

  int size_total = 0;
  int files_total = 1;

  int c = l.size();
  for (int i = 0; i < c; i++)
      {
        QUrl relativeUrl(l[i]);
        QString resolved = baseUrl.resolved(relativeUrl).toString();
        qDebug () << resolved;
        QFileInfo info (resolved);
        if (! info.exists())
           //result += resolved + " " + tr("is not found<br>");
          result += tr ("%1 is not found<br>").arg(resolved);
        else
            {
             //result += resolved + " " + QString::number (info.size() / 1024) + " " + tr("kbytes<br>");
             result += tr ("%1 %2 kbytes<br>").arg(resolved).arg(QString::number (info.size() / 1024));
             size_total += info.size();
             ++files_total;
            }

       }

  result.prepend (tr("Total size = %1 kbytes in %2 files<br>").arg (QString::number (size_total / 1024))
                                                              .arg(QString::number (files_total)));

  log->log (result, "wait01.mng");
}


void rvln::nav_goto_right_tab()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  if (tab_widget->currentIndex() > (tab_widget->count() - 1))
    return;

  tab_widget->setCurrentIndex (tab_widget->currentIndex() + 1);
}


void rvln::nav_goto_left_tab()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  if (tab_widget->currentIndex() == 0)
    return;

  tab_widget->setCurrentIndex (tab_widget->currentIndex() -1);
}

/*
void rvln::check_url_iterate()
{
  qDebug() << "check_url_iterate()";
  qDebug() << sl_urls.first ();

  if (sl_urls.size() == 0)
     {
      qDebug() << "sl_urls.size() == 0";
      return;
     }

  QUrl u = "http://" + sl_urls.first ();

  http_header.setValue ("GET", u.path());
  http_header.setValue("Host", u.host());

  qDebug() << "host " << u.host();
  qDebug() << "path " << u.path();


  http.setHost(u.host());
  http.request(http_header);

  qDebug () << "http.request";

}


void rvln::slot_responseHeaderReceived (const QHttpResponseHeader &resp)
{
  qDebug () << "slot_responseHeaderReceived";

  if (resp.toString().contains("404 Not Found"))
     qDebug () << sl_urls.first() + " 404 Not Found";
  else
     qDebug () << sl_urls.first() + " ok";

  sl_urls.removeFirst();
  check_url_iterate();
}
*/

void rvln::fn_filter_rm_less_than()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText ();
  QStringList l;

  QStringList sl = s.split (QChar::ParagraphSeparator);

  for (int i = 0; i < sl.size(); ++i)
      if (sl[i].size() >= fte->text().toInt())
         l.append (sl[i]);

  d->textEdit->textCursor().insertText (l.join("\n"));
}


void rvln::fn_filter_rm_greater_than()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText ();
  QStringList l;

  QStringList sl = s.split (QChar::ParagraphSeparator);

  for (int i = 0; i < sl.size(); ++i)
      if (sl[i].size() <= fte->text().toInt())
         l.append (sl[i]);

  d->textEdit->textCursor().insertText (l.join("\n"));
}


void rvln::fn_filter_rm_duplicates()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

//  pb_status->show();

  QString s = d->textEdit->textCursor().selectedText ();
  QStringList l;

  QStringList sl = s.split (QChar::ParagraphSeparator);

//  pb_status->setRange (0, sl.size());

  for (int i = 0; i < sl.size(); ++i)
      {
  //     pb_status->setValue (i);
       if (! l.contains (sl[i]))
           l.append (sl[i]);
    //   qApp->processEvents();
      }

  //pb_status->hide();

  d->textEdit->textCursor().insertText (l.join("\n"));
}


void rvln::fn_filter_rm_empty()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText ();
  QStringList l;

  QStringList sl = s.split (QChar::ParagraphSeparator);

  for (int i = 0; i < sl.size(); ++i)
      if (! sl[i].isEmpty())
         l.append (sl[i]);

  d->textEdit->textCursor().insertText (l.join("\n"));
}

/*
void rvln::fn_suck_words()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QHash <QString, QString> h = hash_load (fname_compdict);
  QStringList l = d->get_words();

  int c = l.size();
  if (c == 0)
     return;

  QString st;

  for (int i = 0; i < c; i++)
      {
       st = l[i];
       h.insert (st, st);
      }

  QStringList sl = h.keys();

  qstring_save (fname_compdict, sl.join ("\n"));
}
*/

void rvln::fn_extract_words()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QStringList w = d->get_words();

  CDocument *nd = documents->create_new();
  nd->textEdit->textCursor().insertText (w.join("\n"));
}


void rvln::nav_toggle_hs()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  if (! file_exists (d->file_name))
      return;

  documents->open_file (toggle_fname_header_source (d->file_name), d->charset);
}


QString morse_from_lang (const QString &s, const QString &lang)
{

  QHash<QString, QString> h = hash_load_keyval (":/text-data/morse-" + lang);

  QString result;

  int c = s.size();
  for (int i = 0; i < c; i++)
      {
       QString t = h.value (QString (s[i]));
       if (! t.isNull() || ! t.isEmpty())
          result.append (t + " ");
      }

  return result;
}


QString morse_to_lang (const QString &s, const QString &lang)
{
  QHash<QString, QString> h = hash_load_keyval (":/text-data/morse-" + lang);

  QStringList sl = s.toUpper().split (" ");

  QString result;

  int c = sl.size();
  for (int i = 0; i < c; i++)
      {
       QString t = h.key (sl[i]);
       if (! t.isNull() || ! t.isEmpty())
          result.append (t);
      }

  return result;
}


void rvln::fn_morse_from_en()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (morse_from_lang (d->textEdit->textCursor().selectedText().toUpper(), "en"));
}


void rvln::fn_morse_to_en()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (morse_to_lang (d->textEdit->textCursor().selectedText(), "en"));
}


void rvln::fn_morse_from_ru()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (morse_from_lang (d->textEdit->textCursor().selectedText().toUpper(), "ru"));
}


void rvln::fn_morse_to_ru()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (morse_to_lang (d->textEdit->textCursor().selectedText(), "ru"));
}


void rvln::nav_focus_to_fte()
{
  fte->setFocus (Qt::OtherFocusReason);
}


void rvln::nav_focus_to_editor()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->setFocus (Qt::OtherFocusReason);
}


void rvln::edit_copy_current_fname()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QApplication::clipboard()->setText (d->file_name);
}


void rvln::fn_insert_date()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (QDate::currentDate ().toString (settings->value("date_format", "dd/MM/yyyy").toString()));
}


void rvln::fn_insert_time()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (QTime::currentTime ().toString (settings->value("time_format", "hh:mm:ss").toString()));
}


void rvln::fn_rm_formatting_at_each_line()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString s = d->textEdit->textCursor().selectedText ();
  QStringList l;

  QStringList sl = s.split (QChar::ParagraphSeparator);

  for (int i = 0; i < sl.size(); ++i)
       l.append (sl[i].simplified());

  d->textEdit->textCursor().insertText (l.join("\n"));
}


void rvln::fn_number_arabic_to_roman()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (arabicToRoman (d->textEdit->textCursor().selectedText().toUInt()));
}


void rvln::fn_number_roman_to_arabic()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (QString::number(romanToDecimal (d->textEdit->textCursor().selectedText().toUtf8().data())));
}


void rvln::help_show_gpl()
{
  CDocument *d = documents->open_file (":/COPYING", "UTF-8");
  d->textEdit->setReadOnly (true);
//  inf->change_movie ("you.mng");
}


void rvln::update_dyn_menus()
{
  update_templates();
  update_snippets();
  update_scripts();
}


void rvln::file_open_bookmarks_file()
{
  documents->open_file (fname_bookmarks, "UTF-8");
}


void rvln::file_open_programs_file()
{
#ifdef Q_WS_WIN

  if (! file_exists (fname_programs))
     qstring_save (fname_programs, tr("#external programs list. example:\nopera=\"C:\Program Files\Opera\opera.exe\" \"%s\""));

#else

  if (! file_exists (fname_programs))
     qstring_save (fname_programs, tr("#external programs list. example:\nopera=opera %s"));

#endif

  CDocument *d = documents->open_file (fname_programs, "UTF-8");
}


void rvln::file_open_program()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QAction *a = qobject_cast<QAction *>(sender());
  QString command = programs.value(a->text());
  if (command.isNull() || command.isEmpty())
     return;

  command = command.replace ("%s", d->file_name);
//  qDebug () << command;

  QProcess *process = new QProcess (this);
  process->start (command);
}


void rvln::update_programs()
{
  if (! file_exists (fname_programs))
     return;

  programs = hash_load_keyval (fname_programs);
  if (programs.count() < 0)
     return;

  menu_programs->clear();

  create_menu_from_list (this, menu_programs,
                         programs.keys(),
                         SLOT (file_open_program()));
}


void rvln::fn_insert_template_html()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (qstring_load(":/text-data/template-html"));
}


void rvln::view_hide_error_marks()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QTextCursor cr = d->textEdit->textCursor();

//delete all underlines
  cr.setPosition (0);
  cr.movePosition (QTextCursor::End, QTextCursor::KeepAnchor);
  QTextCharFormat f = cr.blockCharFormat ();
  f.setFontUnderline (false);
  cr.mergeCharFormat (f);
  d->textEdit->document()->setModified (false);
}


void rvln::fn_rm_formatting()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  d->textEdit->textCursor().insertText (d->textEdit->textCursor().selectedText().simplified());
}


void rvln::view_toggle_fs()
{
  setWindowState(windowState() ^ Qt::WindowFullScreen);
}


void rvln::help_show_news()
{
  QString fname = ":/NEWS";
  if (QLocale::system().name().left(2) == "ru")
     fname = ":/NEWS-RU";

  CDocument *d = documents->open_file (fname, "UTF-8");
  d->textEdit->setReadOnly (true);
//  inf->change_movie ("hypno01.mng");
}


void rvln::help_show_changelog()
{
  CDocument *d = documents->open_file (":/ChangeLog", "UTF-8");
  d->textEdit->setReadOnly (true);
//  inf->change_movie ("shoot01.mng");
}


void rvln::help_show_todo()
{
  CDocument *d = documents->open_file (":/TODO", "UTF-8");
  d->textEdit->setReadOnly (true);
//  inf->change_movie ("scary01.mng");
}


void CAboutWindow::closeEvent(QCloseEvent *event)
{
  event->accept();
}


CAboutWindow::CAboutWindow()
{
  setAttribute (Qt::WA_DeleteOnClose);

  QStringList sl_t = qstring_load (":/AUTHORS").split ("##");

  QLabel *logo = new QLabel ();
  logo->setPixmap (QPixmap (":/images/tea_logo.jpg"));
  QTabWidget *tw = new QTabWidget (this);

  QTextEdit *page_code = new QTextEdit ();
  QTextEdit *page_thanks = new QTextEdit ();
  QTextEdit *page_translators = new QTextEdit ();
  QTextEdit *page_maintainers = new QTextEdit ();

  if (sl_t.size() == 4)
     {
      page_code->setPlainText (sl_t[0].trimmed());
      page_thanks->setPlainText (sl_t[3].trimmed());
      page_translators->setPlainText (sl_t[1].trimmed());
      page_maintainers->setPlainText (sl_t[2].trimmed());
     }

  tw->addTab (page_code, tr("Code"));
  tw->addTab (page_thanks, tr("Acknowledgements"));
  tw->addTab (page_translators, tr("Translations"));
  tw->addTab (page_maintainers, tr("Packages"));

  QVBoxLayout *layout = new QVBoxLayout();

  layout->addWidget(logo);
  layout->addWidget(tw);

  setLayout (layout);
  setWindowTitle (tr("About"));
}


void rvln::cb_script_finished (int exitCode, QProcess::ExitStatus exitStatus)
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  qDebug() << "cb_script_finished";


  QString s = qstring_load (fname_tempfile);
  if (! s.isNull() || ! s.isEmpty())
     d->textEdit->textCursor().insertText(s);

  QFile f (fname_tempfile);
  f.remove();
  f.setFileName (fname_tempparamfile);
  f.remove();
}


void rvln::update_scripts()
{
  menu_fn_scripts->clear ();
  QStringList l = read_dir_entries (dir_scripts);

  create_menu_from_list (this, menu_fn_scripts,
                         l,
                         SLOT (fn_run_script()));
}


void rvln::fn_run_script()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  if (! d->textEdit->textCursor().hasSelection ())
     return;

  qDebug() << "hi";

  QAction *a = qobject_cast<QAction *>(sender());
  QString intrp;
  QString fname = dir_scripts + "/" + a->text();
  QString ext = file_get_ext (fname);

  if (ext == "rb")
     intrp = "ruby ";
  else
  if (ext == "py")
     intrp = "python ";
  else
  if (ext == "pl")
     intrp = "perl ";
  else
  if (ext == "sh")
     intrp = "sh ";

  if (intrp.isEmpty())
      return;

  qstring_save (fname_tempfile, d->textEdit->textCursor().selectedText());
  qstring_save (fname_tempparamfile, fte->text());

  QString command = intrp + fname + " " + fname_tempfile + " " + fname_tempparamfile;
  qDebug() << command;

  QProcess *process = new QProcess (this);
  connect(process, SIGNAL(finished ( int, QProcess::ExitStatus )), this, SLOT(cb_script_finished (int, QProcess::ExitStatus )));

  process->start (command);
}


void rvln::fman_nav (const QString &path)
{
  QModelIndex i = mvl_model->index(path);
  if (! i.isValid())
     return;

  tv_fman->setCurrentIndex(i);
  //tv_fman->setExpanded (i, true);
  tv_fman->expand (i);
  tv_fman->scrollTo (i, QAbstractItemView::PositionAtTop);
  ed_fman_path->setText (path);
}


void rvln::fman_item_expanded ( const QModelIndex & index )
{
  ed_fman_path->setText (mvl_model->filePath (index));
}


void rvln::cb_button_saves_as()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  if (ed_fman_fname->text().isEmpty())
     return;

  QString dirname;

  QModelIndex index = tv_fman->currentIndex();
  if (mvl_model->isDir(index))
     dirname = mvl_model->filePath(index);
  else
      dirname = get_file_path (mvl_model->filePath(index));

  QString filename = dirname + "/" + ed_fman_fname->text();

  if (file_exists (filename))
     {
      int ret = QMessageBox::warning (this, "TEA",
                                      tr("%1 already exists\n"
                                      "Do you want to overwrite?")
                                      .arg(filename),
                                       QMessageBox::Yes | QMessageBox::Default,
                                       QMessageBox::Cancel | QMessageBox::Escape);

       if (ret == QMessageBox::Cancel)
         return;
      }

   d->save_with_name (filename, cb_fman_codecs->currentText());
   d->set_markup_mode();

   add_to_last_used_charsets (cb_fman_codecs->currentText());

   d->set_hl ();
   QFileInfo f (d->file_name);
   dir_last = f.path();
   update_dyn_menus();
   mvl_model->refresh();
   main_tab_widget->setCurrentIndex (0);
}


void rvln::fman_home()
{
  fman_nav (QDir::homePath());
}


void rvln::fman_add_bmk()
{
  sl_places_bmx.prepend (ed_fman_path->text());
  qstring_save (fname_places_bookmarks, sl_places_bmx.join("\n"));
  update_places_bookmarks();
}


void rvln::fman_del_bmk()
{
  int i = lv_places->currentRow();
  if (i < 3)
     return;

  QString s = lv_places->item(i)->text();
  if (s.isNull() || s.isEmpty())
     return;

  i = sl_places_bmx.indexOf (s);
  sl_places_bmx.removeAt (i);
  qstring_save (fname_places_bookmarks, sl_places_bmx.join("\n"));
  update_places_bookmarks();
}


void rvln::fman_naventry_confirm()
{
  fman_nav (ed_fman_path->text());
}


void rvln::fman_item_activated (const QModelIndex &index)
{
  if (mvl_model->isDir (index))
     {
      ed_fman_path->setText (mvl_model->filePath (index));
      return;
     }

  QString path = mvl_model->filePath(index);

  if (is_image (path))
     {
      CDocument *d = documents->get_current();
      if (! d)
       return;

      QFileInfo inf (d->file_name);
      QDir dir (inf.absolutePath());

      QImage img;
      img.load (path);
      QString result;

      if (d->markup_mode == "HTML")
         QTextStream(&result) << "<img src=\"" << dir.relativeFilePath (path) <<
                                  "\" border=\"0\" alt=\"\" width=\"" << img.width() << "\" height=\"" << img.height() << "\">";
      else
      if (d->markup_mode == "XHTML")
         QTextStream(&result) << "<img src=\"" << dir.relativeFilePath (path) <<
                                 "\" border=\"0\" alt=\"\" width=\"" << img.width() << "\" height=\"" << img.height() << "\" />";
      else
      if (d->markup_mode == "Docbook")
          QTextStream(&result) << "<mediaobject><imageobject>\n<imagedata fileref=\"" << dir.relativeFilePath (path) <<
                                  "\"/>\n</imageobject></mediaobject>";
      else
      if (d->markup_mode == "LaTeX")
         result = "\\includegraphics{"+ dir.relativeFilePath (path) + "}";

      d->textEdit->textCursor().insertText (result);
      main_tab_widget->setCurrentIndex (0);

      return;
     }


  CDocument *d = documents->open_file (path, cb_fman_codecs->currentText());
  if (d)
     {
      dir_last = get_file_path (d->file_name);
      charset = d->charset;
     }

  add_to_last_used_charsets (cb_fman_codecs->currentText());
  main_tab_widget->setCurrentIndex (0);
}


void rvln::cb_fman_currentChanged ( const QModelIndex & current, const QModelIndex & previous )
{
  QString path = mvl_model->filePath(current);
  if (! mvl_model->isDir (current))
      {
       ed_fman_fname->setText (mvl_model->fileName(current));

       if (is_image (path))
          {
           QPixmap pm (path);
           if (pm.height() > l_fman_preview->height())
              l_fman_preview->setPixmap (pm.scaled(w_places->width(), 200, Qt::KeepAspectRatio));
           else
               l_fman_preview->setPixmap (pm);

           return;
          }

       return;
      }
}


void rvln::fman_places_itemActivated (QListWidgetItem *item)
{
  int index = lv_places->currentRow();
  if (index == 0)
     {
      fman_nav (dir_templates);
      return;
     }

  if (index == 1)
     {
      fman_nav (dir_snippets);
      return;
     }

  if (index == 2)
     {
      fman_nav (dir_scripts);
      return;
     }

  fman_nav (item->text());
}


void rvln::update_places_bookmarks()
{
  lv_places->clear();
  QStringList sl_items;
  sl_items << tr("templates");
  sl_items << tr("snippets");
  sl_items << tr("scripts");
  lv_places->addItems (sl_items);

  if (! file_exists (fname_places_bookmarks))
     return;

  sl_places_bmx = qstring_load (fname_places_bookmarks).split("\n");
  if (sl_places_bmx.size() != 0)
     lv_places->addItems (sl_places_bmx);
}


void rvln::fman_open()
{
  QString f = ed_fman_fname->text();
  if (f[0] == '/')
     {
      CDocument *d = documents->open_file (f, cb_fman_codecs->currentText());
      if (d)
         {
          dir_last = get_file_path (d->file_name);
          charset = d->charset;
          add_to_last_used_charsets (cb_fman_codecs->currentText());
         }

      main_tab_widget->setCurrentIndex (0);
      return;
     }

  QModelIndexList indexes = tv_fman->selectionModel()->selectedIndexes();

  if (indexes.size() == 0)
     return;

  if (indexes.size() == 1)
    {
     //handle the name from the text entry
     QString fname = ed_fman_path->text() + "/" + f;
     CDocument *d = documents->open_file (fname, cb_fman_codecs->currentText());
     if (d)
         {
          dir_last = get_file_path (d->file_name);
          charset = d->charset;
          add_to_last_used_charsets (cb_fman_codecs->currentText());
         }

     main_tab_widget->setCurrentIndex (0);
     return;
    }

  foreach (QModelIndex index, indexes)
          {
           if (index.column() == 0)
              {
               CDocument *d = 0;
               if (! mvl_model->isDir (index))
                  d = documents->open_file (mvl_model->filePath (index), cb_fman_codecs->currentText());
               if (d)
                  {
                   dir_last = get_file_path (d->file_name);
                   charset = d->charset;
                  }
              }
          }

  add_to_last_used_charsets (cb_fman_codecs->currentText());
  main_tab_widget->setCurrentIndex (0);
}


bool CTvFmanKeyPressEater::eventFilter(QObject *obj, QEvent *event)
{
             if (event->type() == QEvent::KeyPress) {
                 QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
                 //qDebug("Ate key press %d", keyEvent->key());
                 if (keyEvent->key() == Qt::Key_Delete)
                    {
                     //QModelIndex index = tv_fman->currentIndex();
                     //if (mvl_model->isDir(index))
                     emit delPressed();
                        return false;


                     //qDebug << mvl_model->filePath(index);

                    }

                 return false;
             } else {
                 // standard event processing
                 return QObject::eventFilter(obj, event);
             }
}


void rvln::fman_del_pressed ()
{
  QModelIndex index = tv_fman->currentIndex();
  if (mvl_model->isDir(index))
      return;

  QString fname = mvl_model->filePath(index);

  int ret = QMessageBox::warning (this, "TEA",
                                      tr("Are you sure to delete\n"
                                      "%1?")
                                      .arg(fname),
                                       QMessageBox::Yes | QMessageBox::Default,
                                       QMessageBox::No | QMessageBox::Escape);

  if (ret == QMessageBox::No)
     return;

  mvl_model->remove (index);
   update_dyn_menus();

  //qDebug() << mvl_model->filePath(index);
}


void rvln::fman_ed_commitData ( QWidget *editor )
{
  update_dyn_menus();
}


void rvln::fman_create_dir()
{
  if (ed_fman_path->text().isEmpty())
     return;

  QString dname = ed_fman_path->text();
  QDir d;
  if (! d.mkpath (dname))
     return;

  mvl_model->refresh();
  fman_nav (dname);
}

#ifndef Q_WS_WIN

void rvln::createFman()
{
  QWidget *wd_fman = new QWidget (this);

  QVBoxLayout *lav_main = new QVBoxLayout ();


  QHBoxLayout *lah_controls = new QHBoxLayout ();
  QHBoxLayout *lah_topbar = new QHBoxLayout ();

  QLabel *l_t = new QLabel (tr("Name"));
  ed_fman_fname = new QLineEdit;

  ed_fman_path = new QLineEdit;
  connect(ed_fman_path, SIGNAL(returnPressed()), this, SLOT(fman_naventry_confirm()));

  tb_fman_dir = new QToolBar;
  tb_fman_dir->setObjectName ("tb_fman_dir");

  QAction *act_fman_go = new QAction(QIcon(":/icons/go.svg"), tr("Go"), this);
  connect(act_fman_go, SIGNAL(triggered()), this, SLOT(fman_naventry_confirm()));

  QAction *act_fman_home = new QAction(QIcon(":/icons/home.svg"), tr("Home"), this);
  connect(act_fman_home, SIGNAL(triggered()), this, SLOT(fman_home()));

  QAction *act_fman_refresh = new QAction(QIcon(":/icons/refresh.svg"), tr("Refresh"), this);

  QAction *act_fman_createdir = new QAction(QIcon(":/icons/create-dir.svg"), tr("Create new directory"), this);
  connect(act_fman_createdir, SIGNAL(triggered()), this, SLOT(fman_create_dir()));


  tb_fman_dir->addAction (act_fman_go);
  tb_fman_dir->addAction (act_fman_home);
  tb_fman_dir->addAction (act_fman_refresh);
  tb_fman_dir->addAction (act_fman_createdir);

  lah_topbar->addWidget (ed_fman_path);
  lah_topbar->addWidget (tb_fman_dir);

  lah_controls->addWidget (l_t);
  lah_controls->addWidget (ed_fman_fname);

  l_t = new QLabel (tr("Charset"));
  cb_fman_codecs = new QComboBox;

  if (sl_last_used_charsets.size () > 0)
     cb_fman_codecs->addItems (sl_last_used_charsets + sl_charsets);
  else
     {
      cb_fman_codecs->addItems (sl_charsets);
      cb_fman_codecs->setCurrentIndex (sl_charsets.indexOf ("UTF-8"));
     }


  QPushButton *bt_fman_open = new QPushButton (tr("Open"), this);
  connect(bt_fman_open, SIGNAL(clicked()), this, SLOT(fman_open()));


  QPushButton *bt_fman_save_as = new QPushButton (tr("Save as"), this);
  connect(bt_fman_save_as, SIGNAL(clicked()), this, SLOT(cb_button_saves_as()));

  lah_controls->addWidget(l_t);
  lah_controls->addWidget(cb_fman_codecs);

  lah_controls->addWidget(bt_fman_open);
  lah_controls->addWidget(bt_fman_save_as);


  mvl_model = new QDirModel;
  mvl_model->setReadOnly (false);
  mvl_model->setFilter (QDir::AllEntries | QDir::NoDotAndDotDot | QDir::Hidden);
  mvl_model->setSorting (QDir::DirsFirst);

  connect(act_fman_refresh, SIGNAL(triggered()), mvl_model, SLOT(refresh()));


  tv_fman = new QTreeView;
  tv_fman->setUniformRowHeights (true);
  tv_fman->setEditTriggers (QAbstractItemView::SelectedClicked);

  CTvFmanKeyPressEater *flt = new CTvFmanKeyPressEater;
  tv_fman->installEventFilter(flt);
  connect(flt, SIGNAL(delPressed()), this, SLOT(fman_del_pressed()));

  QAbstractItemDelegate *ditem = tv_fman->itemDelegate();

  connect(ditem, SIGNAL(commitData ( QWidget *)), this, SLOT(fman_ed_commitData ( QWidget *)));


/*
void QWidget::customContextMenuRequested ( const QPoint & pos )   [signal]
This signal is emitted when the widget's contextMenuPolicy is Qt::CustomContextMenu, and the user has requested a context menu on the widget. The position pos is the position of the context menu event that the widget receives. Normally this is in widget coordinates. The exception to this rule is QAbstractScrollArea and its subclasses that map the context menu event to coordinates of the viewport() .
*/

  connect(tv_fman, SIGNAL(activated ( const QModelIndex &)), this, SLOT(fman_item_activated ( const QModelIndex &)));
  connect(tv_fman, SIGNAL(expanded ( const QModelIndex &)), this, SLOT(fman_item_expanded ( const QModelIndex &)));

  tv_fman->setSelectionMode (QAbstractItemView::ExtendedSelection);
  tv_fman->setSelectionBehavior (QAbstractItemView::SelectItems);
  tv_fman->setModel(mvl_model);
  tv_fman->setRootIndex(mvl_model->index("/"));
  tv_fman->header()->setResizeMode(QHeaderView::ResizeToContents);

  QItemSelectionModel *ism = tv_fman->selectionModel();
  connect(ism, SIGNAL(currentChanged ( const QModelIndex &, const QModelIndex &)), this, SLOT(cb_fman_currentChanged ( const QModelIndex &, const QModelIndex &)));

  QSplitter *w_right = new QSplitter;
  w_right->setOrientation (Qt::Vertical);

  w_places = new QGroupBox (tr("Bookmarks"));
  w_places->setFlat (true);

  QHBoxLayout *lah_places_bar = new QHBoxLayout ();
//  lah_places_bar->setContentsMargins (0, 0, 0, 0);
  QPushButton *bt_add_bmk = new QPushButton ("+");
  QPushButton *bt_del_bmk = new QPushButton ("-");
  lah_places_bar->addWidget (bt_add_bmk);
  lah_places_bar->addWidget (bt_del_bmk);

  connect(bt_add_bmk, SIGNAL(clicked()), this, SLOT(fman_add_bmk()));
  connect(bt_del_bmk, SIGNAL(clicked()), this, SLOT(fman_del_bmk()));


  lv_places = new QListWidget;
  update_places_bookmarks();
  connect(lv_places, SIGNAL(itemActivated (QListWidgetItem *)), this, SLOT(fman_places_itemActivated (QListWidgetItem *)));


  QVBoxLayout *vbox = new QVBoxLayout;
  vbox->addLayout (lah_places_bar);
  vbox->addWidget (lv_places);
  w_places->setLayout (vbox);

  w_right->addWidget (w_places);
  l_fman_preview = new QLabel (tr("Preview"));
  l_fman_preview->setAlignment (Qt::AlignHCenter);
  w_right->addWidget (l_fman_preview);


  spl_fman = new QSplitter(this);
  spl_fman->addWidget(tv_fman);
  spl_fman->addWidget(w_right);

  lav_main->addLayout (lah_topbar);
  lav_main->addLayout (lah_controls);
  lav_main->addWidget (spl_fman);

  wd_fman->setLayout (lav_main);

  main_tab_widget->addTab (wd_fman, tr("browse"));
//  spl_fman->restoreState (settings->value("spl_fman").toByteArray());
}
#endif


void rvln::activate()
{
        setWindowState(windowState() & ~Qt::WindowMinimized);
        raise();
        activateWindow();
}


void rvln::handleMessage(const QString& message)
{
  QString filename (message);
  if (filename.isEmpty() || !QFile::exists(filename))
     return;

  QFileInfo f (filename);

  if (! f.isAbsolute())
     {
      QString fullname = QDir::currentPath() + "/" + filename;
      documents->open_file (fullname, "UTF-8");
     }
  else
      documents->open_file (filename, "UTF-8");

  activate();
}

/*
bool CKeybSound::eventFilter(QObject *obj, QEvent *event)
{
  if (event->type() == QEvent::KeyPress)
     {
       QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
                 qDebug("Ate key press %d", keyEvent->key());

    if (mediaObject->state() != Phonon::PlayingState)
   {
    mediaObject->setCurrentSource( Phonon::MediaSource ("/home/rox/devel/sndsclicks/indr.wav"));

//           mediaObject->enqueue (Phonon::MediaSource ("/home/rox/devel/sndsclicks/click44.wav"));
           mediaObject->play();
 }
                 if (keyEvent->key() == Qt::Key_Delete)
                    {
                 //    return false;

                   }

                 return false;
             } else
                 // standard event processing
                 return QObject::eventFilter(obj, event);
}


CKeybSound::CKeybSound()
{

    audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
    mediaObject = new Phonon::MediaObject(this);
    Phonon::createPath(mediaObject, audioOutput);
}

CKeybSound::~CKeybSound()
{

}
*/


void rvln::fn_convert_quotes()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QString source = d->textEdit->textCursor().selectedText();
  QString c1 = "\xAB";
  QString c2 = "\xBB";
  QString x;
  QString dest;

  bool flag = true;
  int c = source.size() - 1;
  for (int i = 0; i <= c; i++)
     {
      if (source[i] == '\"')
         {
          if (flag)
             x = c1;
          else
              x = c2;

          flag = ! flag;
          dest += x;
         }
      else
          dest += source[i];
     }


  d->textEdit->textCursor().insertText (dest);
}

//from http://www.qt-apps.org/content/show.php/Lightscreen?content=72757&PHPSESSID=22b51715a528addc3a597e4d50f01615
QPixmap getActiveWindow()
{
#ifdef Q_WS_WIN
  HWND fWindow = GetForegroundWindow();

  if (fWindow == null)
    return QPixmap();

  if (fWindow == GetDesktopWindow())
    return QPixmap::grabWindow(QApplication::desktop()->winId());

  return OS::grabWindow(GetForegroundWindow());
//QWidget * QDesktopWidget::screen ( int screen = -1 )
#else
  return QPixmap::grabWindow(QApplication::desktop()->winId());
#endif
}


void rvln::fn_enum()
{
  CDocument *d = documents->get_current();
  if (! d)
     return;

  QStringList source = d->textEdit->textCursor().selectedText().split(QChar::ParagraphSeparator);

  int pad = 0;
  int end = source.size() - 1;
  int step = 1;
  QString result;
  QString prefix;

  QStringList params = fte->text().split("~");

  if (params.size() > 0)
     step = params[0].toInt();

  if (params.size() > 1)
     pad = params[1].toInt();

  if (params.size() > 2)
     prefix = params[2];

  qDebug() << "pad = " << pad << "; step = " << step << "; prefix = " << prefix;

  for (int c = 0; c <= end; c++)
      {

       QString n;
       n = n.setNum (((c + 1) * step));
       if (pad != 0)
          n = n.rightJustified (pad, '0');

      result += n + prefix + source[c] + '\n';
     }

  d->textEdit->textCursor().insertText (result);
}


void rvln::view_stay_on_top()
{
  Qt::WindowFlags flags = windowFlags();
  flags ^= Qt::WindowStaysOnTopHint;
  setWindowFlags( flags );
  show();
  activateWindow();
}


void rvln::update_sessions()
{
  menu_file_sessions->clear ();
  QStringList l = read_dir_entries (dir_sessions);

  create_menu_from_list (this, menu_file_sessions,
                         l,
                         SLOT (file_open_session()));
}


void rvln::file_open_session()
{
  QAction *a = qobject_cast<QAction *>(sender());
  QString s = dir_sessions + "/" + a->text();
  documents->load_from_session (s);
}


void rvln::session_save_as()
{
  if (documents->list.size() < 0)
     return;

  QString f = fte->text();
  if (f.isNull() || f.isEmpty())
      return;

  QString fname = dir_sessions + "/" + f;
  documents->save_to_session (fname);
  update_sessions();
}


void rvln::test()
{
/*QPixmap p = getActiveWindow();
QLabel *l = new QLabel ("t");
l->setPixmap (p);
l->show();
*/

}
