/*******************************************************************************
SingleApplication provides a way to detect a running instance.
Copyright (C) 2008  Ritt K.

*   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SINGLE_APPLICATION_H
#define SINGLE_APPLICATION_H

#include <QtCore/QObject>

#ifdef SINGLEAPPLICATION_DLL
#	ifdef SINGLEAPPLICATION_BUILD
#		define SINGLEAPPLICATION_EXPORT Q_DECL_EXPORT
#	else
#		define SINGLEAPPLICATION_EXPORT Q_DECL_IMPORT
#	endif
#endif
#ifndef SINGLEAPPLICATION_EXPORT
#	define SINGLEAPPLICATION_EXPORT
#endif

class QString;

class SingleApplicationPrivate;
class SINGLEAPPLICATION_EXPORT SingleApplication : public QObject
{
	Q_OBJECT

public:
	explicit SingleApplication(const QString& id, QObject* parent = 0);
	explicit SingleApplication(const QString& id, const QString& serverName, QObject* parent = 0);
	~SingleApplication();

public:
	QString id() const;
	QString serverName() const;

	bool isRunning() const;

	static bool isRunning(const QString& id);
	static bool sendMessage(const QString& id, const QString& message, int timeout = 5000);

public slots:
	bool sendMessage(const QString& message, int timeout = 5000);

signals:
	void messageReceived(const QString& message);

private:
	Q_DISABLE_COPY(SingleApplication);

	SingleApplicationPrivate* d;
};

#endif // SINGLE_APPLICATION_H
