/*******************************************************************************
SingleApplication provides a way to detect a running instance.
Copyright (C) 2008  Ritt K.
*   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef SYSTEMSEMAPHORE_H
#define SYSTEMSEMAPHORE_H

#include <QtCore/QString>

#ifdef SINGLEAPPLICATION_DLL
#	ifdef SINGLEAPPLICATION_BUILD
#		define SINGLEAPPLICATION_EXPORT Q_DECL_EXPORT
#	else
#		define SINGLEAPPLICATION_EXPORT Q_DECL_IMPORT
#	endif
#endif
#ifndef SINGLEAPPLICATION_EXPORT
#	define SINGLEAPPLICATION_EXPORT
#endif

class SystemSemaphorePrivate;
class SINGLEAPPLICATION_EXPORT SystemSemaphore
{
public:
	enum AccessMode
	{
		Open,
		Create
	};

	enum SystemSemaphoreError
	{
		NoError,
		PermissionDenied,
		KeyError,
		AlreadyExists,
		NotFound,
		OutOfResources,
		UnknownError
	};

	SystemSemaphore(const QString& key, int initialValue = 0, AccessMode mode = Open);
	~SystemSemaphore();

	void setKey(const QString& key, int initialValue = 0, AccessMode mode = Open);
	QString key() const;

	bool acquire();
	bool tryAcquire(int timeout);
	bool release(int n = 1);

	SystemSemaphoreError error() const;
	QString errorString() const;

private:
	Q_DISABLE_COPY(SystemSemaphore)

	SystemSemaphorePrivate* d;
};

#endif // SYSTEMSEMAPHORE_H
