/***************************************************************************
 *   Copyright (C) 2007-2008 by Peter Semiletov                            *
 *   peter.semiletov@gmail.com                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/***************************************************************************
some code is taken from Scribus::util.cpp:
                             -------------------
    begin                : Fri Sep 14 2001
    copyright            : (C) 2001 by Franz Schmid
    email                : Franz.Schmid@altmuehlnet.de
 ***************************************************************************/
/* roman.c by Adam Rogoyski (apoc@laker.net) Temperanc on EFNet irc
 * Copyright (C) 1998 Adam Rogoyski
 * Converts Decimal numbers to Roman Numerals and Roman Numberals to
 * Decimals on the command line or in Interactive mode.
 * Uses an expanded Roman Numeral set to handle numbers up to 999999999

*/

#include <QtGui>


QString get_file_path (const QString &fileName)
{
  QFileInfo f (fileName);
  return f.path();
}


bool file_exists (const QString &fileName)
{
  if (fileName.isNull() || fileName.isEmpty())
     return false;
  QFile file (fileName);
  return file.exists();
}


QString qstring_load (const QString &fileName)
{
  QFile file (fileName);

  QString s;

  if (! file.open (QFile::ReadOnly | QFile::Text))
       return s;

  QTextStream in(&file);
  in.setCodec ("UTF-8");

  s = in.readAll();
  return s;
}


void qstring_list_print (const QStringList &l)
{
  int c = l.size();
  for (int i = 0; i < c; i++)
      qDebug() << l[i];
}


bool qstring_save (const QString &fileName, const QString &data)
{
  QFile file (fileName);
  if (! file.open (QFile::WriteOnly | QFile::Text))
      return false;

  QTextStream out (&file);

  out.setCodec ("UTF-8");

  out << data;

  return true;
}


void create_menu_from_list (QObject *handler,
                            QMenu *menu,
                            const QStringList &list,
                            const char *method
                           )
{
  int c = list.size();
  for (int i = 0; i < c; ++i)
      {
       QAction *act = new QAction (list[i], menu->parentWidget());
       handler->connect (act, SIGNAL(triggered()), handler, method);
       menu->addAction (act);
      }
}


QString mod_to_string (Qt::KeyboardModifiers k)
{
  QString s;

  if ( k & Qt::ShiftModifier )
     s += "Shift+";

  if ( k & Qt::ControlModifier )
     s += "Ctrl+";

  if ( k & Qt::AltModifier )
     s += "Alt+";

  if ( k & Qt::MetaModifier )
     s+= "Meta+";

  return s;
}


QString keycode_to_string (int k)
{
 // return QKeySequence(k).toString();

  QString s;
  s = QChar (k);

  if (k == Qt::Key_F1)
     s = "F1";

  if (k == Qt::Key_F2)
     s = "F2";

  if (k == Qt::Key_F3)
     s = "F3";

  if (k == Qt::Key_4)
     s = "F4";

  if (k == Qt::Key_F5)
     s = "F5";

  if (k == Qt::Key_F6)
     s = "F6";

  if (k == Qt::Key_F7)
     s = "F7";

  if (k == Qt::Key_F8)
     s = "F8";

  if (k == Qt::Key_F9)
     s = "F9";

  if (k == Qt::Key_F10)
     s = "F10";

  if (k == Qt::Key_F11)
     s = "F11";

  if (k == Qt::Key_F12)
     s = "F12";

  return s;
}


QStringList read_dir_entries (const QString &path)
{
  QDir dir;
  dir.setPath (path);
  return dir.entryList (QDir::AllEntries | QDir::NoDotAndDotDot);
}


QString file_get_ext (const QString &file_name)
{
  QFileInfo fi = QFileInfo (file_name);

  if (! fi.exists())
      return QString ("");

  return fi.suffix().toLower();
}


QStringList html_get_by_patt (const QString &s, const QString &spatt)
{
  QStringList result;

  int c = s.size();
  int i = 0;

  while (i < c)
        {
         int start = s.indexOf (spatt, i, Qt::CaseInsensitive);

         if (start == -1)
             break;

         int end = s.indexOf ('"', start + spatt.size());
         if (end == -1)
             break;

         result.prepend (s.mid (start + spatt.size(), (end - start) - spatt.size()));

         i = end + 1;
        }

  return result;
}


QHash<QString, QString> stringlist_to_hash (const QStringList &l)
{
  QHash<QString, QString> result;
  if (l.empty())
     return result;

  int c = l.size();
  QString st;

  for (int i = 0; i < c; i++)
      {
       st = l[i];
       result.insert (st, st);
      }

  return result;
}


QHash<QString, QString> hash_load (const QString &fname)
{
  QHash<QString, QString> result;
  if (! file_exists (fname))
     return result;

  result = stringlist_to_hash (qstring_load (fname).split ("\n"));

  return result;
}


QString toggle_fname_header_source (const QString &fileName)
{
  QFileInfo f (fileName);

  QString ext = f.suffix ();

  if (ext == "c" || ext == "cpp" || ext == "cxx" || ext == "cc" || ext == "c++")
     {
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".h"))
        return f.absolutePath() + "/" + f.baseName () + ".h";
      else
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".hxx"))
        return f.absolutePath() + "/" + f.baseName () + ".hxx";
      else
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".h++"))
        return f.absolutePath() + "/" + f.baseName () + ".h++";
      else
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".hh"))
        return f.absolutePath() + "/" + f.baseName () + ".hh";
     }

  if (ext == "h" || ext == "cpp" || ext == "cxx" || ext == "cc" || ext == "c++")
     {
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".c"))
        return f.absolutePath() + "/" + f.baseName () + ".c";
      else
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".cpp"))
        return f.absolutePath() + "/" + f.baseName () + ".cpp";
      else
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".cxx"))
        return f.absolutePath() + "/" + f.baseName () + ".cxx";
      else
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".c++"))
        return f.absolutePath() + "/" + f.baseName () + ".c++";
      else
      if (file_exists (f.absolutePath() + "/" + f.baseName () + ".cc"))
        return f.absolutePath() + "/" + f.baseName () + ".cc";
     }

  return fileName;
}


QHash<QString, QString> hash_load_keyval (const QString &fname)
{
  QHash<QString, QString> result;
  if (! file_exists (fname))
     return result;

  QStringList l = qstring_load (fname).split ("\n");

  int c = l.size();
  for (int i = 0; i < c; i++)
      {
       QStringList sl = l[i].split("=");
       if (sl.size() == 2)
          result.insert (sl[0], sl[1]);
      }

  return result;
}


bool is_image (const QString &filename)
{
  if (filename.endsWith (".jpg" ,Qt::CaseInsensitive)  ||
      filename.endsWith (".jpeg" ,Qt::CaseInsensitive) ||
      filename.endsWith (".png" ,Qt::CaseInsensitive)  ||
      filename.endsWith (".gif" ,Qt::CaseInsensitive)  ||
      filename.endsWith (".png" ,Qt::CaseInsensitive)  ||
      filename.endsWith (".svg" ,Qt::CaseInsensitive)  ||
      filename.endsWith (".tiff" ,Qt::CaseInsensitive) ||
      filename.endsWith (".tif" ,Qt::CaseInsensitive)
     )
     return true;

  return false;
}


//this code is taken from Scribus::util.cpp:
const QString arabicToRoman (uint i)
{
        QString roman("");
        int arabic = i;

        while (arabic - 1000000 >= 0){
        roman += "m";
        arabic -= 1000000;
        }
        while (arabic - 900000 >= 0){
        roman += "cm";
        arabic -= 900000;
        }
        while (arabic - 500000 >= 0){
        roman += "d";
        arabic -= 500000;
        }
        while (arabic - 400000 >= 0){
        roman += "cd";
        arabic -= 400000;
        }
        while (arabic - 100000 >= 0){
        roman += "c";
        arabic -= 100000;
        }
        while (arabic - 90000 >= 0){
        roman += "xc";
        arabic -= 90000;
        }
        while (arabic - 50000 >= 0){
        roman += "l";
        arabic -= 50000;
        }
        while (arabic - 40000 >= 0){
        roman += "xl";
        arabic -= 40000;
        }
        while (arabic - 10000 >= 0){
        roman += "x";
        arabic -= 10000;
        }
        while (arabic - 9000 >= 0){
        roman += "Mx";
        arabic -= 9000;
        }
        while (arabic - 5000 >= 0){
        roman += "v";
        arabic -= 5000;
        }
        while (arabic - 4000 >= 0){
        roman += "Mv";
        arabic -= 4000;
        }
        while (arabic - 1000 >= 0){
        roman += "M";
        arabic -= 1000;
        }
        while (arabic - 900 >= 0){
        roman += "CM";
        arabic -= 900;
        }
        while (arabic - 500 >= 0){
        roman += "D";
        arabic -= 500;
        }
        while (arabic - 400 >= 0){
        roman += "CD";
        arabic -= 400;
        }
        while (arabic - 100 >= 0){
        roman += "C";
        arabic -= 100;
        }
        while (arabic - 90 >= 0){
        roman += "XC";
        arabic -= 90;
        }
        while (arabic - 50 >= 0){
        roman += "L";
        arabic -= 50;
        }
        while (arabic - 40 >= 0){
        roman += "XL";
        arabic -= 40;
        }
        while (arabic - 10 >= 0){
        roman += "X";
        arabic -= 10;
        }
        while (arabic - 9 >= 0){
        roman += "IX";
        arabic -= 9;
        }
        while (arabic - 5 >= 0){
        roman += "V";
        arabic -= 5;
        }
        while (arabic - 4 >= 0){
        roman += "IV";
        arabic -= 4;
        }
        while (arabic - 1 >= 0){
        roman += "I";
        arabic -= 1;
        }
        return roman;
}


/*
from:

* roman.c by Adam Rogoyski (apoc@laker.net) Temperanc on EFNet irc
 * Copyright (C) 1998 Adam Rogoyski
 * Converts Decimal numbers to Roman Numerals and Roman Numberals to
 * Decimals on the command line or in Interactive mode.
 * Uses an expanded Roman Numeral set to handle numbers up to 999999999


*/
#define I 1
#define V 5
#define X 10
#define L 50
#define C 100
#define D 500
#define M 1000
#define P 5000
#define Q 10000
#define R 50000
#define S 100000
#define T 500000
#define U 1000000
#define B 5000000
#define W 10000000
#define N 50000000
#define Y 100000000
#define Z 500000000

static long
value (char c)
{
   switch (c)
   {
      case 'I':
         return I;
      case 'V':
         return V;
      case 'X':
         return X;
      case 'L':
         return L;
      case 'C':
         return C;
      case 'D':
         return D;
      case 'M':
         return M;
      case 'P':
         return P;
      case 'Q':
         return Q;
      case 'R':
         return R;
      case 'S':
         return S;
      case 'T':
         return T;
      case 'U':
         return U;
      case 'B':
         return B;
      case 'W':
         return W;
      case 'N':
         return N;
      case 'Y':
         return Y;
      case 'Z':
         return Z;
      default:
         return 0;
   }
}


long romanToDecimal (const char *roman)
{
   long decimal = 0;
   for (; *roman; roman++)
   {
      /* Check for four of a letter in a fow */
      if ((*(roman + 1) && *(roman + 2) && *(roman + 3))
         && (*roman == *(roman + 1))
         && (*roman == *(roman + 2))
         && (*roman == *(roman + 3)))
         return 0;
      /* Check for two five type numbers */
      if (  ((*roman == 'V') && (*(roman + 1) == 'V'))
         || ((*roman == 'L') && (*(roman + 1) == 'L'))
         || ((*roman == 'D') && (*(roman + 1) == 'D'))
         || ((*roman == 'P') && (*(roman + 1) == 'P'))
         || ((*roman == 'R') && (*(roman + 1) == 'R'))
         || ((*roman == 'T') && (*(roman + 1) == 'T'))
         || ((*roman == 'B') && (*(roman + 1) == 'B'))
         || ((*roman == 'N') && (*(roman + 1) == 'N'))
         || ((*roman == 'Z') && (*(roman + 1) == 'Z')))
         return 0;
      /* Check for two lower characters before a larger one */
      if ((value(*roman) == value(*(roman + 1))) && (*(roman + 2))
         && (value(*(roman + 1)) < value(*(roman + 2))))
         return 0;
      /* Check for the same character on either side of a larger one */
      if ((*(roman + 1) && *(roman + 2))
         && (value(*roman) == value(*(roman + 2)))
         && (value(*roman) < value(*(roman + 1))))
         return 0;
      /* Check for illegal nine type numbers */
      if (!strncmp(roman, "LXL", 3) || !strncmp(roman, "DCD", 3)
       || !strncmp(roman, "PMP", 3) || !strncmp(roman, "RQR", 3)
       || !strncmp(roman, "TST", 3) || !strncmp(roman, "BUB", 3)
       || !strncmp(roman, "NWN", 3) || !strncmp(roman, "VIV", 3))
         return 0;
      if (value(*roman) < value(*(roman + 1)))
      {
         /* check that subtracted value is at least 10% larger,
            i.e. 1990 is not MXM, but MCMXC */
         if ((10 * value(*roman)) < value(*(roman + 1)))
            return 0;
         /* check for double subtraction, i.e. IVX */
         if (value(*(roman + 1)) <= value(*(roman + 2)))
            return 0;
         /* check for subtracting by a number starting with a 5
            ie.  VX, LD LM */
         if (*roman == 'V' || *roman == 'L' || *roman == 'D'
          || *roman == 'P' || *roman == 'R' || *roman == 'T'
          || *roman == 'B' || *roman == 'N')
            return 0;
         decimal += value (*(roman + 1)) - value (*roman);
         roman++;
      }
      else
      {
         decimal += value (*roman);
      }
   }
   return decimal;
}
